#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QWidget, QVBoxLayout, QPushButton, QHBoxLayout
from helper.usesqapplication import UsesQApplication


class QWidgetTest(UsesQApplication):

    def test_setLayout(self):
        layout = QVBoxLayout()
        btn1 = QPushButton("button_v1")
        layout.addWidget(btn1)

        btn2 = QPushButton("button_v2")
        layout.addWidget(btn2)

        layout2 = QHBoxLayout()

        btn1 = QPushButton("button_h1")
        layout2.addWidget(btn1)

        btn2 = QPushButton("button_h2")
        layout2.addWidget(btn2)

        layout.addLayout(layout2)

        widget = QWidget()
        widget.setLayout(layout)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot
from PySide6.QtWidgets import QDialog
from PySide6.QtBluetooth import (QBluetoothAddress, QBluetoothServiceInfo,
                                 QBluetoothServiceDiscoveryAgent, QBluetoothLocalDevice)

from ui_service import Ui_ServiceDiscovery


class ServiceDiscoveryDialog(QDialog):
    def __init__(self, name, address, parent=None):
        super().__init__(parent)
        self._ui = Ui_ServiceDiscovery()
        self._ui.setupUi(self)

        # Using default Bluetooth adapter
        local_device = QBluetoothLocalDevice()
        adapter_address = QBluetoothAddress(local_device.address())

        # In case of multiple Bluetooth adapters it is possible to
        # set which adapter will be used by providing MAC Address.
        # Example code:
        #
        # adapterAddress = QBluetoothAddress("XX:XX:XX:XX:XX:XX")
        # discoveryAgent = QBluetoothServiceDiscoveryAgent(adapterAddress)

        self._discovery_agent = QBluetoothServiceDiscoveryAgent(adapter_address)
        self._discovery_agent.setRemoteAddress(address)

        self.setWindowTitle(name)

        self._discovery_agent.serviceDiscovered.connect(self.add_service)
        self._discovery_agent.finished.connect(self._ui.status.hide)
        self._discovery_agent.start()

    @Slot(QBluetoothServiceInfo)
    def add_service(self, info):
        line = info.serviceName()
        if not line:
            return

        if info.serviceDescription():
            line += "\n\t" + info.serviceDescription()
        if info.serviceProvider():
            line += "\n\t" + info.serviceProvider()
        self._ui.list.addItem(line)



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import sys
import argparse
import logging
import shutil
import traceback
from pathlib import Path
from textwrap import dedent

from deploy_lib import (create_config_file, cleanup, config_option_exists, PythonExecutable,
                        MAJOR_VERSION, HELP_EXTRA_IGNORE_DIRS, HELP_EXTRA_MODULES)
from deploy_lib.android import AndroidData, AndroidConfig
from deploy_lib.android.buildozer import Buildozer


""" pyside6-android-deploy deployment tool

    Deployment tool that uses buildozer (https://buildozer.readthedocs.io/en/latest/) and
    python-for-android (https://python-for-android.readthedocs.io/en/latest/) to deploy PySide6
    applications to Android

    How does it work?

    Command: pyside6-android-deploy --wheel-pyside=<pyside_wheel_path>
                                    --wheel-shiboken=<shiboken_wheel_path>
                                    --ndk-path=<optional_ndk_path>
                                    --sdk-path=<optional_sdk_path>
             pyside6-android-deploy android -c /path/to/pysidedeploy.spec


    Note: If --ndk-path and --sdk-path are not specified, the cache of the tool
    `.pyside6_android_deploy` is checked in the user's HOME directory. If it is not found, the user
    will have to manually download them.

    Prerequisities: Python main entrypoint file should be named "main.py"

    Platforms Supported: aarch64, armv7a, i686, x86_64

    Config file:
        On the first run of the tool, it creates a config file called pysidedeploy.spec which
        controls the various characteristic of the deployment. Users can simply change the value
        in this config file to achieve different properties ie. change the application name,
        deployment platform etc.

        Note: This file is used by both pyside6-deploy and pyside6-android-deploy
"""


def main(name: str = None, pyside_wheel: Path = None, shiboken_wheel: Path = None,
         ndk_path: Path = None, sdk_path: Path = None, config_file: Path = None, init: bool = False,
         loglevel=logging.WARNING, dry_run: bool = False, keep_deployment_files: bool = False,
         force: bool = False, extra_ignore_dirs: str = None, extra_modules_grouped: str = None):

    logging.basicConfig(level=loglevel)

    if extra_ignore_dirs:
        extra_ignore_dirs = extra_ignore_dirs.split(",")

    extra_modules = []
    if extra_modules_grouped:
        tmp_extra_modules = extra_modules_grouped.split(",")
        for extra_module in tmp_extra_modules:
            if extra_module.startswith("Qt"):
                extra_modules.append(extra_module[2:])
            else:
                extra_modules.append(extra_module)

    main_file = Path.cwd() / "main.py"
    if not main_file.exists():
        raise RuntimeError(("[DEPLOY] For Android deployment to work, the main"
                            " entrypoint Python file should be named 'main.py'"
                            " and it should be run from the application"
                            " directory"))

    android_data = AndroidData(wheel_pyside=pyside_wheel, wheel_shiboken=shiboken_wheel,
                               ndk_path=ndk_path, sdk_path=sdk_path)

    python = PythonExecutable(dry_run=dry_run, init=init, force=force)

    config_file_exists = config_file and Path(config_file).exists()

    if config_file_exists:
        logging.info(f"[DEPLOY] Using existing config file {config_file}")
    else:
        config_file = create_config_file(main_file=main_file, dry_run=dry_run)

    config = AndroidConfig(config_file=config_file, source_file=main_file,
                           python_exe=python.exe, dry_run=dry_run, android_data=android_data,
                           existing_config_file=config_file_exists,
                           extra_ignore_dirs=extra_ignore_dirs, name=name)

    if not config.wheel_pyside and not config.wheel_shiboken:
        raise RuntimeError(f"[DEPLOY] No PySide{MAJOR_VERSION} and Shiboken{MAJOR_VERSION} wheels"
                           "found")

    cleanup(config=config, is_android=True)

    python.install_dependencies(config=config, packages="android_packages", is_android=True)

    try:
        config.modules += list(set(extra_modules).difference(set(config.modules)))

        # this cannot be done when config file is initialized because cleanup() removes it
        # so this can only be done after the cleanup()
        config.jars_dir = config.find_jars_dir()
        config.recipe_dir = config.find_recipe_dir()

        # TODO: include qml files from pysidedeploy.spec rather than from extensions
        # buildozer currently includes all the files with .qml extension

        # init buildozer
        Buildozer.dry_run = dry_run
        logging.info("[DEPLOY] Creating buildozer.spec file")
        Buildozer.initialize(pysidedeploy_config=config)

        # writing config file
        if not dry_run:
            config.update_config()

        if init:
            # config file created above. Exiting.
            logging.info(f"[DEPLOY]: Config file  {config.config_file} created")
            return

        # run buildozer
        logging.info("[DEPLOY] Running buildozer deployment")
        Buildozer.create_executable(config.mode)

        # move buildozer build files to {generated_files_path}
        if not dry_run:
            buildozer_build_dir = config.project_dir / ".buildozer"
            if not buildozer_build_dir.exists():
                logging.info(f"[DEPLOY] Unable to copy {buildozer_build_dir} to "
                             f"{config.generated_files_path}. {buildozer_build_dir} does not exist")
            logging.info(f"[DEPLOY] copy {buildozer_build_dir} to {config.generated_files_path}")
            shutil.move(buildozer_build_dir, config.generated_files_path)

        logging.info(f"[DEPLOY] apk created in {config.exe_dir}")
    except Exception:
        print(f"Exception occurred: {traceback.format_exc()}")
    finally:
        if config.generated_files_path and config and not keep_deployment_files:
            cleanup(config=config, is_android=True)

    logging.info("[DEPLOY] End")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=dedent(f"""
                           This tool deploys PySide{MAJOR_VERSION} to Android platforms.

                           Note: The main python entrypoint should be named main.py
                           """),
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument("-c", "--config-file", type=lambda p: Path(p).absolute(),
                        default=(Path.cwd() / "pysidedeploy.spec"),
                        help="Path to the .spec config file")

    parser.add_argument(
        "--init", action="store_true",
        help="Create pysidedeploy.spec file, if it doesn't already exists")

    parser.add_argument(
        "-v", "--verbose", help="run in verbose mode", action="store_const",
        dest="loglevel", const=logging.INFO)

    parser.add_argument("--dry-run", action="store_true", help="show the commands to be run")

    parser.add_argument("--keep-deployment-files", action="store_true",
                        help="keep the generated deployment files generated")

    parser.add_argument("-f", "--force", action="store_true", help="force all input prompts")

    parser.add_argument("--name", type=str, help="Application name")

    parser.add_argument("--wheel-pyside", type=lambda p: Path(p).resolve(),
                        help=f"Path to PySide{MAJOR_VERSION} Android Wheel",
                        required=not config_option_exists())

    parser.add_argument("--wheel-shiboken", type=lambda p: Path(p).resolve(),
                        help=f"Path to shiboken{MAJOR_VERSION} Android Wheel",
                        required=not config_option_exists())

    parser.add_argument("--ndk-path", type=lambda p: Path(p).resolve(),
                        help=("Path to Android NDK. The required version is r26b."
                              "If not provided, the tool will check its cache at "
                              ".pyside6_android_deploy to find the NDK.")
                        )

    parser.add_argument("--sdk-path", type=lambda p: Path(p).resolve(),
                        help=("Path to Android SDK. If omitted, the tool's cache at "
                              ".pyside6_android_deploy is checked to find the SDK. Otherwise "
                              "the default from buildozer is used.")
                        )

    parser.add_argument("--extra-ignore-dirs", type=str, help=HELP_EXTRA_IGNORE_DIRS)

    parser.add_argument("--extra-modules", type=str, help=HELP_EXTRA_MODULES)

    args = parser.parse_args()

    # check if the Python version is greater than 3.12
    if sys.version_info >= (3, 12):
        raise RuntimeError("[DEPLOY] Android deployment requires Python version 3.11 or lower. "
                           "This is due to a restriction in buildozer.")

    main(args.name, args.wheel_pyside, args.wheel_shiboken, args.ndk_path, args.sdk_path,
         args.config_file, args.init, args.loglevel, args.dry_run, args.keep_deployment_files,
         args.force, args.extra_ignore_dirs, args.extra_modules)



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from PySide6.QtGui import QPainter, QBrush, QColor
from PySide6.QtQml import QmlElement
from PySide6.QtCore import QPointF, Qt, Property, Signal
from PySide6.QtQuick import QQuickPaintedItem

QML_IMPORT_NAME = "TextBalloon"
QML_IMPORT_MAJOR_VERSION = 1
QML_IMPORT_MINOR_VERSION = 0  # Optional


@QmlElement
class TextBalloon(QQuickPaintedItem):

    rightAlignedChanged = Signal()

    def __init__(self, parent=None):
        self._rightAligned = False
        super().__init__(parent)

    @Property(bool, notify=rightAlignedChanged)
    def rightAligned(self):
        return self._rightAligned

    @rightAligned.setter
    def rightAligned(self, value):
        self._rightAligned = value
        self.rightAlignedChanged.emit()

    def paint(self, painter: QPainter):

        brush = QBrush(QColor("#007430"))

        painter.setBrush(brush)
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)

        itemSize = self.size()

        painter.drawRoundedRect(0, 0, itemSize.width(), itemSize.height() - 10, 10, 10)

        if self.rightAligned:
            points = [
                QPointF(itemSize.width() - 10.0, itemSize.height() - 10.0),
                QPointF(itemSize.width() - 20.0, itemSize.height()),
                QPointF(itemSize.width() - 30.0, itemSize.height() - 10.0),
            ]
        else:
            points = [
                QPointF(10.0, itemSize.height() - 10.0),
                QPointF(20.0, itemSize.height()),
                QPointF(30.0, itemSize.height() - 10.0),
            ]
        painter.drawConvexPolygon(points)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from PySide6.QtCore import QObject
from testbinding import TestObject


class Receiver(QObject):
    def __init__(self):
        super().__init__()
        self.called = False

    def ReceiverDecorator(func):
        def decoratedFunction(self, *args, **kw):
            func(self, *args, **kw)
        return decoratedFunction

    # This method with the same name of the internal decorated function
    # is here to test the binding capabilities.
    def decoratedFunction(self):
        pass

    @ReceiverDecorator
    def slot(self):
        self.called = True


class DecoratedSlotTest(unittest.TestCase):

    def testCallingOfDecoratedSlot(self):
        obj = TestObject(0)
        receiver = Receiver()
        obj.staticMethodDouble.connect(receiver.slot)
        obj.emitStaticMethodDoubleSignal()
        self.assertTrue(receiver.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import os
import sys
from pathlib import Path
from PySide6.QtCore import QUrl
from PySide6.QtGui import QGuiApplication, QSurfaceFormat
from PySide6.QtQml import QQmlApplicationEngine

from PySide6.QtQuick3D import QQuick3D

if __name__ == "__main__":
    app = QGuiApplication(sys.argv)

    QSurfaceFormat.setDefaultFormat(QQuick3D.idealSurfaceFormat(4))

    engine = QQmlApplicationEngine()
    qml_file = os.fspath(Path(__file__).resolve().parent / 'main.qml')
    engine.load(QUrl.fromLocalFile(qml_file))
    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QFile, QFileInfo, QDir
from PySide6.QtWidgets import QApplication
from PySide6.QtSvgWidgets import QSvgWidget


class QSvgWidgetTest(unittest.TestCase):

    def testLoad(self):
        directory = os.path.dirname(__file__)
        tigerPath = QDir.cleanPath(f"{directory}/../QtSvg/tiger.svg")
        self.assertTrue(QFileInfo.exists(tigerPath))

        app = QApplication([])  # noqa: F841
        fromFile = QSvgWidget()
        fromFile.load(tigerPath)
        self.assertTrue(fromFile.renderer().isValid())

        tigerFile = QFile(tigerPath)
        tigerFile.open(QFile.OpenModeFlag.ReadOnly)
        tigerData = tigerFile.readAll()
        fromContents = QSvgWidget()
        fromContents.load(tigerData)
        self.assertTrue(fromContents.renderer().isValid())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
regenerate_example_resources.py
===============================

Regenerates the QRC resource files of the PySide examples.
"""


import subprocess
import sys
from pathlib import Path

RCC_COMMAND = "pyside6-rcc"
LRELEASE_COMMAND = "lrelease"


def prepare_linguist_example(path):
    """Create the .qm files for the Linguist example which are bundled in the QRC file"""
    translations_dir = path / "translations"
    if not translations_dir.is_dir():
        translations_dir.mkdir(parents=True)

    for ts_file in path.glob("*.ts"):
        qm_file = translations_dir / f"{ts_file.stem}.qm"
        print("Regenerating ", ts_file, qm_file)
        ex = subprocess.call([LRELEASE_COMMAND, ts_file, "-qm", qm_file])
        if ex != 0:
            print(f"{LRELEASE_COMMAND} failed for {ts_file}", file=sys.stderr)
            sys.exit(ex)


def generate_rc_file(qrc_file):
    """Regenerate the QRC resource file."""
    dir = qrc_file.parent
    if dir.name == "linguist":
        prepare_linguist_example(dir)

    target_file = dir / f"{qrc_file.stem}_rc.py"
    if not target_file.is_file():  # prefix naming convention
        target_file2 = qrc_file.parent / f"rc_{qrc_file.stem}.py"
        if target_file2.is_file():
            target_file = target_file2
    if not target_file.is_file():
        print(target_file, " does not exist.", file=sys.stderr)
        return

    print("Regenerating ", qrc_file, target_file)
    ex = subprocess.call([RCC_COMMAND, qrc_file, "-o", target_file])
    if ex != 0:
        print(f"{RCC_COMMAND} failed for {qrc_file}", file=sys.stderr)
        sys.exit(ex)


if __name__ == '__main__':
    examples_path = Path(__file__).resolve().parent.parent / "examples"
    for qrc_file in examples_path.glob("**/*.qrc"):
        generate_rc_file(qrc_file)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for parent-child relationship'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QRegularExpression, QTimer, Qt


class TestObject1(QTimer):
    def __init(self, parent):
        super().__init__(parent)


class TestObject2(TestObject1):
    def __init(self, parent):
        super().__init__(parent)


class ParentRefCountCase(unittest.TestCase):
    '''Test case for the refcount changes of setParent'''

    def setUp(self):
        # Acquire resources
        self.parent = QObject()
        self.child = QObject()

    def tearDown(self):
        # Release resources
        del self.child
        del self.parent
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testSetParent(self):
        # QObject.setParent() refcount changes
        self.assertEqual(sys.getrefcount(self.child), 2)
        self.child.setParent(self.parent)
        self.assertEqual(sys.getrefcount(self.child), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testSetParentTwice(self):
        self.assertEqual(sys.getrefcount(self.child), 2)
        self.child.setParent(self.parent)
        self.assertEqual(sys.getrefcount(self.child), 3)
        self.child.setParent(self.parent)
        self.assertEqual(sys.getrefcount(self.child), 3)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testConstructor(self):
        # QObject(QObject) refcount changes
        child = QObject(self.parent)
        expected_ref_count = 2 if sys.version_info >= (3, 14) else 3
        self.assertEqual(sys.getrefcount(child), expected_ref_count)


class ParentCase(unittest.TestCase):
    '''Small collection of tests related to parent-child relationship'''

    def testSetParent(self):
        # QObject.setParent()
        parent = QObject()
        child = QObject()
        child.setParent(parent)

        self.assertEqual(parent, child.parent())

    def testParentConstructor(self):
        # QObject(parent)
        parent = QObject()
        child = QObject(parent)

        self.assertEqual(parent, child.parent())

        orig_repr = repr(child)
        del child
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(orig_repr, repr(parent.children()[0]))

    def testChildren(self):
        # QObject.children()
        parent = QObject()
        children = [QObject(parent) for x in range(25)]

        self.assertEqual(parent.children(), children)

    def testFindChild(self):
        # QObject.findChild() with all QObject
        parent = QObject()
        children = [QObject(parent) for i in range(20)]

        for i, child in enumerate(children):
            child.setObjectName(f'object{i}')

        for i, child in enumerate(children):
            self.assertEqual(child, parent.findChild(QObject, f'object{i}'))

    def testFindChildOptions(self):
        parent = QObject()
        child = QObject(parent)
        nested_child_name = 'nestedChild'
        nested_child = QObject(child)
        nested_child.setObjectName(nested_child_name)

        search_result = parent.findChild(QObject, nested_child_name)
        self.assertTrue(search_result)
        search_result = parent.findChild(QObject, nested_child_name,
                                         Qt.FindChildOption.FindChildrenRecursively)
        self.assertTrue(search_result)
        search_result = parent.findChild(QObject, nested_child_name,
                                         Qt.FindChildOption.FindDirectChildrenOnly)
        self.assertFalse(search_result)

        search_results = parent.findChildren(QObject, nested_child_name)
        self.assertEqual(len(search_results), 1)
        search_result = parent.findChildren(QObject, nested_child_name,
                                            Qt.FindChildOption.FindChildrenRecursively)
        self.assertEqual(len(search_results), 1)
        search_results = parent.findChildren(QObject, nested_child_name,
                                             Qt.FindChildOption.FindDirectChildrenOnly)
        self.assertEqual(len(search_results), 0)

    def testFindChildWithoutName(self):
        parent = QObject()
        children = [QObject(parent) for i in range(20)]

        for i, child in enumerate(children):
            child.setObjectName(f'object{i}')

        child = parent.findChild(QObject)
        self.assertTrue(isinstance(child, QObject))

    def testFindChildren(self):
        # QObject.findChildren() with all QObject
        parent = QObject()
        target_name = 'foo'
        children = [QTimer(parent) for i in range(20)]
        children.extend([QObject(parent) for i in range(20)])

        for i, child in enumerate(children):
            if i % 5 == 0:
                child.setObjectName(target_name)
            else:
                child.setObjectName(str(i))

        # Emulates findChildren with the intended outcome
        target_children = [x for x in children if x.objectName() == target_name]
        test_children = parent.findChildren(QObject, target_name)
        self.assertEqual(target_children, test_children)

        # test findChildren default value
        res = parent.findChildren(QTimer)
        self.assertEqual(len(res), 20)

        # test findChildren with a QRegularExpression
        res = parent.findChildren(QObject, QRegularExpression("^fo+"))
        self.assertEqual(res, test_children)

    def testParentEquality(self):
        # QObject.parent() == parent
        parent = QObject()
        child = QObject(parent)
        self.assertEqual(parent, child.parent())

    def testFindChildByType(self):
        parent = QObject()
        expected = TestObject2(parent)
        actual = parent.findChild(TestObject2)
        self.assertEqual(actual, expected)
        actual = parent.findChild(TestObject1)
        self.assertEqual(actual, expected)
        actual = parent.findChild(QTimer)
        self.assertEqual(actual, expected)

    def testFindChildrenByType(self):
        parent = QObject()
        expected = [TestObject2(parent)]
        actual = parent.findChildren(TestObject2)
        self.assertEqual(actual, expected)
        actual = parent.findChildren(TestObject1)
        self.assertEqual(actual, expected)
        actual = parent.findChildren(QTimer)
        self.assertEqual(actual, expected)


class TestParentOwnership(unittest.TestCase):
    '''Test case for Parent/Child object ownership'''

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testParentDestructor(self):
        parent = QObject()
        base_ref_count_parent = sys.getrefcount(parent)

        child = QObject(parent)
        base_ref_count_child = sys.getrefcount(child)
        self.assertEqual(sys.getrefcount(parent), base_ref_count_parent)

        del parent
        self.assertEqual(sys.getrefcount(child), base_ref_count_child - 1)

        # this will fail because parent deleted child cpp object
        self.assertRaises(RuntimeError, lambda: child.objectName())

    # test parent with multiples children
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testMultipleChildren(self):
        o = QObject()
        base_ref_count_o = sys.getrefcount(o)

        c = QObject(o)
        base_ref_count_c = sys.getrefcount(c)
        self.assertEqual(sys.getrefcount(o), base_ref_count_o)

        c2 = QObject(o)
        self.assertEqual(sys.getrefcount(o), base_ref_count_o)
        self.assertEqual(sys.getrefcount(c), base_ref_count_c)
        self.assertEqual(sys.getrefcount(c2), base_ref_count_c)

        del o
        self.assertEqual(sys.getrefcount(c), base_ref_count_c - 1)
        self.assertEqual(sys.getrefcount(c2), base_ref_count_c - 1)

        # this will fail because parent deleted child cpp object
        self.assertRaises(RuntimeError, lambda: c.objectName())
        self.assertRaises(RuntimeError, lambda: c2.objectName())

    # test recursive parent
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRecursiveParent(self):
        o = QObject()
        base_ref_count_o = sys.getrefcount(o)

        c = QObject(o)
        base_ref_count_c = sys.getrefcount(c)
        self.assertEqual(sys.getrefcount(o), base_ref_count_o)

        c2 = QObject(c)
        base_ref_count_c2 = sys.getrefcount(c2)
        self.assertEqual(sys.getrefcount(o), base_ref_count_o)
        self.assertEqual(sys.getrefcount(c), base_ref_count_c)

        del o
        self.assertEqual(sys.getrefcount(c), base_ref_count_c - 1)
        self.assertEqual(sys.getrefcount(c2), base_ref_count_c2 - 1)

        # this will fail because parent deleted child cpp object
        self.assertRaises(RuntimeError, lambda: c.objectName())
        self.assertRaises(RuntimeError, lambda: c2.objectName())

    # test parent transfer
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testParentTransfer(self):
        o = QObject()
        base_ref_count = sys.getrefcount(o)

        c = QObject()
        self.assertEqual(sys.getrefcount(c), base_ref_count)

        c.setParent(o)
        self.assertEqual(sys.getrefcount(c), base_ref_count + 1)

        c.setParent(None)
        self.assertEqual(sys.getrefcount(c), base_ref_count)

        del c
        del o


class ExtQObject(QObject):
    def __init__(self):
        super().__init__()


class ReparentingTest(unittest.TestCase):
    '''Test cases for reparenting'''

    def testParentedQObjectIdentity(self):
        object_list = []
        parent = QObject()
        for i in range(3):
            obj = QObject()
            object_list.append(obj)
            obj.setParent(parent)
        for child in parent.children():
            self.assertTrue(child in object_list)

    def testParentedExtQObjectType(self):
        object_list = []
        parent = QObject()
        for i in range(3):
            obj = ExtQObject()
            object_list.append(obj)
            obj.setParent(parent)
        for orig, child in zip(object_list, parent.children()):
            self.assertEqual(type(orig), type(child))

    def testReparentedQObjectIdentity(self):
        object_list = []
        old_parent = QObject()
        new_parent = QObject()
        for i in range(3):
            obj = QObject()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for child in new_parent.children():
            self.assertTrue(child in object_list)

    def testReparentedExtQObjectType(self):
        object_list = []
        old_parent = QObject()
        new_parent = QObject()
        for i in range(3):
            obj = ExtQObject()
            object_list.append(obj)
            obj.setParent(old_parent)
        for obj in object_list:
            obj.setParent(new_parent)
        for orig, child in zip(object_list, new_parent.children()):
            self.assertEqual(type(orig), type(child))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the opengl/contextinfo example from Qt v5.x"""

from argparse import ArgumentParser, RawTextHelpFormatter
import numpy
import sys
from textwrap import dedent


from PySide6.QtCore import (QCoreApplication, QLibraryInfo, QSize, QTimer, Qt,
                            Slot)
from PySide6.QtGui import (QMatrix4x4, QOpenGLContext, QSurfaceFormat, QWindow)
from PySide6.QtOpenGL import (QOpenGLBuffer, QOpenGLShader,
                              QOpenGLShaderProgram, QOpenGLVertexArrayObject)
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QMessageBox, QPlainTextEdit,
                               QWidget)
from PySide6.support import VoidPtr
try:
    from OpenGL import GL
except ImportError:
    app = QApplication(sys.argv)
    message_box = QMessageBox(QMessageBox.Critical, "ContextInfo",
                              "PyOpenGL must be installed to run this example.", QMessageBox.Close)
    message_box.setDetailedText("Run:\npip install PyOpenGL PyOpenGL_accelerate")
    message_box.exec()
    sys.exit(1)

vertex_shader_source_110 = dedent("""
    // version 110
    attribute highp vec4 posAttr;
    attribute lowp vec4 colAttr;
    varying lowp vec4 col;
    uniform highp mat4 matrix;
    void main() {
       col = colAttr;
       gl_Position = matrix * posAttr;
    }
    """)

fragment_shader_source_110 = dedent("""
    // version 110
    varying lowp vec4 col;
    void main() {
       gl_FragColor = col;
    }
    """)

vertex_shader_source = dedent("""
    // version 150
    in vec4 posAttr;
    in vec4 colAttr;
    out vec4 col;
    uniform mat4 matrix;
    void main() {
       col = colAttr;
       gl_Position = matrix * posAttr;
    }
    """)

fragment_shader_source = dedent("""
    // version 150
    in vec4 col;
    out vec4 fragColor;
    void main() {
       fragColor = col;
    }
    """)

vertices = numpy.array([0, 0.707, -0.5, -0.5, 0.5, -0.5], dtype=numpy.float32)
colors = numpy.array([1, 0, 0, 0, 1, 0, 0, 0, 1], dtype=numpy.float32)


def print_surface_format(surface_format):
    if surface_format.profile() == QSurfaceFormat.OpenGLContextProfile.CoreProfile:
        profile_name = 'core'
    else:
        profile_name = 'compatibility'
    major = surface_format.majorVersion()
    minor = surface_format.minorVersion()
    return f"{profile_name} version {major}.{minor}"


class RenderWindow(QWindow):
    def __init__(self, fmt):
        super().__init__()
        self.setSurfaceType(QWindow.SurfaceType.OpenGLSurface)
        self.setFormat(fmt)
        self.context = QOpenGLContext(self)
        self.context.setFormat(self.requestedFormat())
        if not self.context.create():
            raise Exception("Unable to create GL context")
        self.program = None
        self.timer = None
        self.angle = 0

    def init_gl(self):
        self.program = QOpenGLShaderProgram(self)
        self.vao = QOpenGLVertexArrayObject()
        self.vbo = QOpenGLBuffer()

        fmt = self.context.format()
        use_new_style_shader = fmt.profile() == QSurfaceFormat.OpenGLContextProfile.CoreProfile
        # Try to handle 3.0 & 3.1 that do not have the core/compatibility profile
        # concept 3.2+ has. This may still fail since version 150 (3.2) is
        # specified in the sources but it's worth a try.
        if (fmt.renderableType() == QSurfaceFormat.RenderableType.OpenGL and fmt.majorVersion() == 3
                and fmt.minorVersion() <= 1):
            use_new_style_shader = not fmt.testOption(QSurfaceFormat.FormatOption.DeprecatedFunctions)  # noqa: E501

        vertex_shader = vertex_shader_source if use_new_style_shader else vertex_shader_source_110
        fragment_shader = (fragment_shader_source
                           if use_new_style_shader
                           else fragment_shader_source_110)
        if not self.program.addShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Vertex, vertex_shader):  # noqa: E501
            log = self.program.log()
            raise Exception("Vertex shader could not be added: {log} ({vertexShader})")
        if not self.program.addShaderFromSourceCode(QOpenGLShader.ShaderTypeBit.Fragment, fragment_shader):  # noqa: E501
            log = self.program.log()
            raise Exception(f"Fragment shader could not be added: {log} ({fragment_shader})")
        if not self.program.link():
            log = self.program.log()
            raise Exception(f"Could not link shaders: {log}")

        self._pos_attr = self.program.attributeLocation("posAttr")
        self._col_attr = self.program.attributeLocation("colAttr")
        self._matrix_uniform = self.program.uniformLocation("matrix")

        self.vbo.create()
        self.vbo.bind()
        self._vertices_data = vertices.tobytes()
        self._colors_data = colors.tobytes()
        vertices_size = 4 * vertices.size
        colors_size = 4 * colors.size
        self.vbo.allocate(VoidPtr(self._vertices_data), vertices_size + colors_size)
        self.vbo.write(vertices_size, VoidPtr(self._colors_data), colors_size)
        self.vbo.release()

        with QOpenGLVertexArrayObject.Binder(self.vao):
            if self.vao.isCreated():  # have VAO support, use it
                self.setup_vertex_attribs()

    def setup_vertex_attribs(self):
        self.vbo.bind()
        self.program.setAttributeBuffer(self._pos_attr, GL.GL_FLOAT, 0, 2)
        self.program.setAttributeBuffer(self._col_attr, GL.GL_FLOAT, 4 * vertices.size, 3)
        self.program.enableAttributeArray(self._pos_attr)
        self.program.enableAttributeArray(self._col_attr)
        self.vbo.release()

    def exposeEvent(self, event):
        if self.isExposed():
            self.render()
            if self.timer is None:
                self.timer = QTimer(self)
                self.timer.timeout.connect(self.slot_timer)
            if not self.timer.isActive():
                self.timer.start(10)
        else:
            if self.timer and self.timer.isActive():
                self.timer.stop()

    def render(self):
        if not self.context.makeCurrent(self):
            raise Exception("makeCurrent() failed")
        functions = self.context.functions()
        if self.program is None:
            functions.glEnable(GL.GL_DEPTH_TEST)
            functions.glClearColor(0, 0, 0, 1)
            self.init_gl()

        retina_scale = self.devicePixelRatio()
        functions.glViewport(0, 0, self.width() * retina_scale,
                             self.height() * retina_scale)
        functions.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

        self.program.bind()
        matrix = QMatrix4x4()
        matrix.perspective(60, 4 / 3, 0.1, 100)
        matrix.translate(0, 0, -2)
        matrix.rotate(self.angle, 0, 1, 0)
        self.program.setUniformValue(self._matrix_uniform, matrix)

        if self.vao.isCreated():
            self.vao.bind()
        else:  # no VAO support, set the vertex attribute arrays now
            self.setup_vertex_attribs()

        functions.glDrawArrays(GL.GL_TRIANGLES, 0, 3)

        self.vao.release()
        self.program.release()

        # swapInterval is 1 by default which means that swapBuffers() will (hopefully) block
        # and wait for vsync.
        self.context.swapBuffers(self)
        self.context.doneCurrent()

    @Slot()
    def slot_timer(self):
        self.render()
        self.angle += 1

    def glInfo(self):
        if not self.context.makeCurrent(self):
            raise Exception("makeCurrent() failed")
        functions = self.context.functions()
        gl_vendor = functions.glGetString(GL.GL_VENDOR)
        gl_renderer = functions.glGetString(GL.GL_RENDERER)
        gl_version = functions.glGetString(GL.GL_VERSION)
        gl_lang_version = functions.glGetString(GL.GL_SHADING_LANGUAGE_VERSION)
        context_surface_format = print_surface_format(self.context.format())
        surface_format = print_surface_format(self.format())

        text = (f"Vendor: {gl_vendor}\n"
                f"Renderer: {gl_renderer}\n"
                f"Version: {gl_version}\n"
                f"Shading language: {gl_lang_version}\n"
                f"Context Format: {context_surface_format}\n\n"
                f"Surface Format: {surface_format}")
        self.context.doneCurrent()
        return text


class MainWindow(QWidget):
    def __init__(self):
        super().__init__()
        h_box_layout = QHBoxLayout(self)
        self._plain_text_edit = QPlainTextEdit()
        self._plain_text_edit.setMinimumWidth(400)
        self._plain_text_edit.setReadOnly(True)
        h_box_layout.addWidget(self._plain_text_edit)
        self._render_window = RenderWindow(QSurfaceFormat())
        container = QWidget.createWindowContainer(self._render_window)
        container.setMinimumSize(QSize(400, 400))
        h_box_layout.addWidget(container)

    def update_description(self):
        build = QLibraryInfo.build()
        gl = self._render_window.glInfo()
        text = f"{build}\n\nPython {sys.version}\n\n{gl}"
        self._plain_text_edit.setPlainText(text)


if __name__ == '__main__':
    parser = ArgumentParser(description="contextinfo", formatter_class=RawTextHelpFormatter)
    parser.add_argument('--gles', '-g', action='store_true',
                        help='Use OpenGL ES')
    parser.add_argument('--software', '-s', action='store_true',
                        help='Use Software OpenGL')
    parser.add_argument('--desktop', '-d', action='store_true',
                        help='Use Desktop OpenGL')
    options = parser.parse_args()
    if options.gles:
        QCoreApplication.setAttribute(Qt.ApplicationAttribute.AA_UseOpenGLES)
    elif options.software:
        QCoreApplication.setAttribute(Qt.ApplicationAttribute.AA_UseSoftwareOpenGL)
    elif options.desktop:
        QCoreApplication.setAttribute(Qt.ApplicationAttribute.AA_UseDesktopOpenGL)

    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.show()
    main_window.update_description()
    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QByteArray concatenation with '+' operator'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QByteArray


class QByteArrayConcatenationOperatorTest(unittest.TestCase):
    '''Test cases for QByteArray concatenation with '+' operator'''

    def testConcatQByteArrayAndPythonString(self):
        # Test concatenation of a QByteArray with a Python bytes, in this order
        qba = QByteArray(bytes('foo', "UTF-8"))
        result = qba + bytes('\x00bar', "UTF-8")
        self.assertEqual(type(result), QByteArray)
        self.assertEqual(result, bytes('foo\x00bar', "UTF-8"))

    def testConcatPythonStringAndQByteArray(self):
        # Test concatenation of a Python bytes with a QByteArray, in this order
        qba = QByteArray(bytes('foo', "UTF-8"))
        result = bytes('bar\x00', "UTF-8") + qba
        self.assertEqual(type(result), QByteArray)
        self.assertEqual(result, bytes('bar\x00foo', "UTF-8"))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""
The gallery example is a simple application with a drawer menu that contains
all the Qt Quick Controls. Each menu item opens a page that shows the
graphical appearance of a control, allows you to interact with the control,
and explains in which circumstances it is handy to use this control.
"""

import os
import sys
import platform

from PySide6.QtGui import QGuiApplication, QIcon
from PySide6.QtCore import QSettings, QUrl
from PySide6.QtQml import QQmlApplicationEngine
from PySide6.QtQuickControls2 import QQuickStyle

import rc_gallery  # noqa: F401

if __name__ == "__main__":
    QGuiApplication.setApplicationName("Gallery")
    QGuiApplication.setOrganizationName("QtProject")

    app = QGuiApplication()
    QIcon.setThemeName("gallery")

    settings = QSettings()
    if not os.environ.get("QT_QUICK_CONTROLS_STYLE"):
        if style_name := settings.value("style"):
            QQuickStyle.setStyle(style_name)

    engine = QQmlApplicationEngine()

    built_in_styles = ["Basic", "Fusion", "Imagine", "Material", "Universal", "FluentWinUI3"]

    if platform.system() == "Darwin":
        built_in_styles.append("macOS")
        built_in_styles.append("iOS")
    elif platform.system() == "Windows":
        built_in_styles.append("Windows")
    engine.setInitialProperties({"builtInStyles": built_in_styles})

    engine.load(QUrl.fromLocalFile(":/gallery.qml"))
    rootObjects = engine.rootObjects()
    if not rootObjects:
        sys.exit(-1)

    window = rootObjects[0]
    window.setIcon(QIcon(':/qt-project.org/logos/pysidelogo.png'))

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

MOUSE_BUTTONS = ("NoButton", "AllButtons", "LeftButton", "RightButton", "MiddleButton",
                 "BackButton", "XButton", "ExtraButton", "ForwardButton", "ExtraButton",
                 "TaskButton")

MODIFIERS = ("NoModifier", "ShiftModifier", "ControlModifier", "AltModifier",
             "MetaModifier", "KeypadModifier", "GroupSwitchModifier")

GLOBAL_COLORS = ("white", "black", "red", "darkRed", "green", "darkGreen", "blue",
                 "darkBlue", "cyan", "darkCyan", "magenta", "darkMagenta", "yellow",
                 "darkYellow", "gray", "darkGray", "lightGray", "transparent")

ALIGN_VALUES = ("AlignHCenter", "AlignLeft", "AlignCenter", "AlignRight", "AlignVCenter",
                "AlignTop", "AlignBotton", "AlignJustify", "AlignBaseline", "AlignAbsolute",
                "AlignLeading", "Trailing")


def _get_replacements():
    result = [
        ("Qt::Key_", "Qt.Key.Key_"),
        ("Qt::CTRL", "Qt.Modifier.CTRL"),
        ("Qt::ALT", "Qt.Modifier.ALT"),
        ("Qt::CaseInsensitive", "Qt.CaseSensitivity.CaseInsensitive"),
        ("Qt::CaseSensitive", "Qt.CaseSensitivity.CaseSensitive"),
        ("QImage::Format_", "QImage.Format.Format_"),
        ("Qt::WA_DeleteOnClose", "Qt.WidgetAttribute.WA_DeleteOnClose"),
        ("QQuickView::Ready", "QQuickView.Status.Ready"),
        ("QQuickView::Error", "QQuickView.Status.Error"),
        ("QQuickView::Loading", "QQuickView.Status.Loading"),
        ("QPainter::Antialiasing", "QPainter.RenderHint.Antialiasing"),
        ("QQuickView::SizeRootObjectToView", "QQuickView.ResizeMode.SizeRootObjectToView"),
        ("QQuickView::SizeViewToRootObject", "QQuickView.ResizeMode.SizeViewToRootObject"),
        ("QKeySequence::", "QKeySequence.StandardKey."),
        ("QEvent::", "QEvent.Type.")
    ]
    for c in GLOBAL_COLORS:
        result.append((f"Qt::{c}", f"Qt.GlobalColor.{c}"))
    for b in ("Close", "Ok", "Cancel", "Yes", "No"):
        result.append((f"QDialogButtonBox::{b}", f"QDialogButtonBox.StandardButton.{b}"))
        result.append((f"QMessageBox::{b}", f"QMessageBox.StandardButton.{b}"))
    for b in MOUSE_BUTTONS:
        result.append((f"Qt::{b}", f"Qt.MouseButton.{b}"))
    for a in ALIGN_VALUES:
        result.append((f"Qt::{a}", f"Qt.AlignmentFlag.{a}"))
    for m in MODIFIERS:
        result.append((f"Qt::{m}", f"Qt.KeyboardModifier.{m}"))
    for m in ("ReadOnly", "WriteOnly", "Text"):
        result.append((f"QIODevice::{m}", f"QIODevice.OpenModeFlag.{m}"))
        result.append((f"QFile::{m}", f"QFile.OpenModeFlag.{m}"))
    for p in ("Preferred", "Ignored", "Fixed", "Maximum", "Minimum", "Expanding"):
        result.append((f"QSizePolicy::{p}", f"QSizePolicy.Policy.{p}"))
    for r in ("DisplayRole", "EditRole"):
        result.append((f"Qt::{r}", f"Qt.ItemDataRole.{r}"))
    for f in ("Box", "StyledPanel", "Panel", "WinPanel", "NoFrame"):
        result.append((f"QFrame::{f}", "QFrame.Shape.{f}"))
    for f in ("Raised", "Sunken"):
        result.append((f"QFrame::{f}", "QFrame.Shadow.{f}"))
    return result


REPLACEMENTS = _get_replacements()


def qualify_enums(s):
    for replacement in REPLACEMENTS:
        s = s.replace(replacement[0], replacement[1])
    return s



#!/usr/bin/python
# Copyright (C) 2025 Ford Motor Company
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for RepFile'''

import os
import sys
import unittest
from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)
from PySide6.QtRemoteObjects import RepFile

from test_shared import wrap_tests_for_cleanup

contents = """
class Simple
{
    PROP(int i = 2);
    PROP(float f = -1. READWRITE);
    SIGNAL(random(int i));
    SLOT(void reset());
};
"""


@wrap_tests_for_cleanup()
class QRepFileConstructor(unittest.TestCase):
    '''Test case for RepFile constructors'''
    expected = "RepFile(Classes: [Simple], PODs: [])"

    def setUp(self):
        '''Set up test environment'''
        self.cwd = Path(__file__).parent
        self.path = self.cwd / "simple.rep"

    def testRepFileFromPath(self):
        '''Test constructing RepFile from a path'''
        with open(self.path, 'r') as f:
            rep_file = RepFile(f.read())
        self.assertEqual(str(rep_file), self.expected)

    def testRepFileFromString(self):
        '''Test constructing RepFile from a string'''
        rep_file = RepFile(contents)
        self.assertEqual(str(rep_file), self.expected)

    def testRepFileInvalidString(self):
        '''Test constructing RepFile from a string'''
        with self.assertRaises(RuntimeError) as result:
            RepFile("\n\n}\n\n")
        self.assertEqual(str(result.exception),
                         "Error parsing input, line 3: error: Unknown token encountered")

    def testRepFileNoArguments(self):
        '''Test constructing RepFile with no arguments'''
        with self.assertRaises(TypeError):
            RepFile()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QStringListModel, Qt
from PySide6.QtWidgets import QAbstractItemView, QApplication, QListView


class TestBug964 (unittest.TestCase):

    def testIt(self):
        app = QApplication([])  # noqa: F841
        model = QStringListModel(["1", "2"])
        view = QListView()
        view.setModel(model)
        view.setCurrentIndex(model.index(0, 0))
        newCursor = view.moveCursor(QAbstractItemView.CursorAction.MoveDown,
                                    Qt.KeyboardModifier.NoModifier)
        self.assertEqual(newCursor.row(), 1)
        self.assertEqual(newCursor.column(), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' PYSIDE-315: Test that all signals and slots of a class (including any mixin classes)
    are registered at type parsing time. Also test that the signal and slot
    indices do not change after signal connection or emission. '''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal, Slot


class Mixin:
    mixinSignal = Signal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class MixinTwo(Mixin):
    mixinTwoSignal = Signal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.mixinTwoSlotCalled = False

    @Slot()
    def mixinTwoSlot(self):
        self.mixinTwoSlotCalled = True


class MixinThree:
    mixinThreeSignal = Signal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.mixinThreeSlotCalled = False

    @Slot()
    def mixinThreeSlot(self):
        self.mixinThreeSlotCalled = True


class Derived(Mixin, QObject):
    derivedSignal = Signal(str)

    def __init__(self):
        super().__init__()
        self.derivedSlotCalled = False
        self.derivedSlotString = ''

    @Slot(str)
    def derivedSlot(self, theString):
        self.derivedSlotCalled = True
        self.derivedSlotString = theString


class MultipleDerived(MixinTwo, MixinThree, Mixin, QObject):
    derivedSignal = Signal(str)

    def __init__(self):
        super().__init__()
        self.derivedSlotCalled = False
        self.derivedSlotString = ''

    @Slot(str)
    def derivedSlot(self, theString):
        self.derivedSlotCalled = True
        self.derivedSlotString = theString


class MixinTest(unittest.TestCase):
    def testMixinSignalSlotRegistration(self):
        obj = Derived()
        m = obj.metaObject()

        # Should contain 2 signals and 1 slot immediately after type parsing
        self.assertEqual(m.methodCount() - m.methodOffset(), 3)

        # Save method indices to check that they do not change
        methodIndices = {}
        for i in range(m.methodOffset(), m.methodCount()):
            signature = m.method(i).methodSignature()
            methodIndices[signature] = i

        # Check derivedSignal emission
        obj.derivedSignal.connect(obj.derivedSlot)
        obj.derivedSignal.emit('emit1')
        self.assertTrue(obj.derivedSlotCalled)
        obj.derivedSlotCalled = False

        # Check derivedSignal emission after mixingSignal connection
        self.outsideSlotCalled = False

        @Slot()
        def outsideSlot():
            self.outsideSlotCalled = True

        obj.mixinSignal.connect(outsideSlot)
        obj.derivedSignal.emit('emit2')
        self.assertTrue(obj.derivedSlotCalled)
        self.assertFalse(self.outsideSlotCalled)
        obj.derivedSlotCalled = False

        # Check mixinSignal emission
        obj.mixinSignal.emit()
        self.assertTrue(self.outsideSlotCalled)
        self.assertFalse(obj.derivedSlotCalled)
        self.outsideSlotCalled = False

        # Check that method indices haven't changed.
        # Make sure to requery for the meta object, to check that a new one was not
        # created as a child of the old one.
        m = obj.metaObject()
        self.assertEqual(m.methodCount() - m.methodOffset(), 3)
        for i in range(m.methodOffset(), m.methodCount()):
            signature = m.method(i).methodSignature()
            self.assertEqual(methodIndices[signature], i)

    def testMixinSignalSlotRegistrationWithMultipleInheritance(self):
        obj = MultipleDerived()
        m = obj.metaObject()

        # Should contain 4 signals and 3 slots immediately after type parsing
        self.assertEqual(m.methodCount() - m.methodOffset(), 7)

        # Save method indices to check that they do not change
        methodIndices = {}
        for i in range(m.methodOffset(), m.methodCount()):
            signature = m.method(i).methodSignature()
            methodIndices[signature] = i

        # Check derivedSignal emission
        obj.derivedSignal.connect(obj.derivedSlot)
        obj.derivedSignal.emit('emit1')
        self.assertTrue(obj.derivedSlotCalled)
        self.assertFalse(obj.mixinTwoSlotCalled)
        self.assertFalse(obj.mixinThreeSlotCalled)
        obj.derivedSlotCalled = False

        # Check derivedSignal emission after mixinThreeSignal connection
        self.outsideSlotCalled = False

        @Slot()
        def outsideSlot():
            self.outsideSlotCalled = True

        obj.mixinThreeSignal.connect(obj.mixinThreeSlot)
        obj.mixinThreeSignal.connect(outsideSlot)
        obj.derivedSignal.emit('emit2')
        self.assertTrue(obj.derivedSlotCalled)
        self.assertFalse(obj.mixinTwoSlotCalled)
        self.assertFalse(obj.mixinThreeSlotCalled)
        self.assertFalse(self.outsideSlotCalled)
        obj.derivedSlotCalled = False

        # Check mixinThreeSignal emission
        obj.mixinThreeSignal.emit()
        self.assertTrue(self.outsideSlotCalled)
        self.assertTrue(obj.mixinThreeSlotCalled)
        self.assertFalse(obj.derivedSlotCalled)
        self.assertFalse(obj.mixinTwoSlotCalled)
        self.outsideSlotCalled = False
        obj.mixinThreeSlotCalled = False

        # Check mixinTwoSignal emission
        obj.mixinTwoSignal.connect(obj.mixinTwoSlot)
        obj.mixinTwoSignal.emit()
        self.assertTrue(obj.mixinTwoSlotCalled)
        self.assertFalse(obj.mixinThreeSlotCalled)
        self.assertFalse(obj.derivedSlotCalled)
        self.assertFalse(self.outsideSlotCalled)
        obj.mixinTwoSlotCalled = False

        # Check mixinSignal emission
        obj.mixinSignal.connect(outsideSlot)
        obj.mixinSignal.emit()
        self.assertTrue(self.outsideSlotCalled)
        self.assertFalse(obj.mixinTwoSlotCalled)
        self.assertFalse(obj.mixinThreeSlotCalled)
        self.assertFalse(obj.derivedSlotCalled)
        self.outsideSlotCalled = False

        # Check that method indices haven't changed.
        # Make sure to requery for the meta object, to check that a new one was not
        # created as a child of the old one.
        m = obj.metaObject()
        self.assertEqual(m.methodCount() - m.methodOffset(), 7)
        for i in range(m.methodOffset(), m.methodCount()):
            signature = m.method(i).methodSignature()
            self.assertEqual(methodIndices[signature], i)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QTimer.singleShot'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QThread, QTimer, Signal, Slot, SLOT
from helper.usesqapplication import UsesQApplication


class WatchDog(QObject):
    '''Exits the QCoreApplication main loop after sometime.'''

    def __init__(self, watched):
        super().__init__()
        self.times_called = 0
        self.watched = watched

    def timerEvent(self, evt):
        self.times_called += 1
        if self.times_called == 20:
            self.watched.exit_app_cb()


class ThreadForContext(QThread):
    def __init__(self):
        super().__init__()
        self.called = False
        self.qthread = None
        self.context = QObject()

    def run(self):
        self.called = True
        self.qthread = QThread.currentThread()
        self.exec()


class TestSingleShot(UsesQApplication):
    '''Test case for QTimer.singleShot'''

    def setUp(self):
        # Acquire resources
        UsesQApplication.setUp(self)
        self.watchdog = WatchDog(self)
        self.called = False
        self.qthread = None

    def tearDown(self):
        # Release resources
        del self.watchdog
        del self.called
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        UsesQApplication.tearDown(self)

    def callback(self):
        self.called = True
        self.qthread = QThread.currentThread()
        self.qthread.exit()
        self.app.quit()

    def testSingleShot(self):
        QTimer.singleShot(100, self.callback)
        self.app.exec()
        self.assertTrue(self.called)

    def testSingleShotZero(self):
        QTimer.singleShot(0, self.callback)
        self.app.exec()
        self.assertTrue(self.called)

    def testSingleShotWithContext(self):
        thread = ThreadForContext()
        thread.start()
        thread.context.moveToThread(thread)
        QTimer.singleShot(100, thread.context, self.callback)
        self.app.exec()
        thread.wait()
        self.assertTrue(self.called)
        self.assertTrue(thread.called)
        self.assertEqual(self.qthread, thread.qthread)

    def testSingleShotWithContextZero(self):
        thread = ThreadForContext()
        thread.start()
        thread.context.moveToThread(thread)
        QTimer.singleShot(0, thread.context, self.callback)
        self.app.exec()
        thread.wait()
        self.assertTrue(self.called)
        self.assertTrue(thread.called)
        self.assertEqual(self.qthread, thread.qthread)


class TestSingleShotCallableObject(UsesQApplication):
    '''Test case for QTimer.singleShot with callable inside an object'''

    def setUp(self):
        # Acquire resources
        UsesQApplication.setUp(self)
        self.watchdog = WatchDog(self)

    def tearDown(self):
        # Release resources
        del self.watchdog
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        UsesQApplication.tearDown(self)

    class CallbackObject(QObject):
        def __init__(self, app) -> None:
            super().__init__()
            self.app = app

        @Slot()
        def func(self):
            self.called = True
            self.app.quit()

    def testSingleShotWithObjectAndMember(self):
        callback = self.CallbackObject(self.app)
        QTimer.singleShot(100, callback, SLOT("func()"))
        self.app.exec()
        self.assertTrue(callback.called)

    def testSingleShotWithObjectAndMemberZero(self):
        callback = self.CallbackObject(self.app)
        QTimer.singleShot(0, callback, SLOT("func()"))
        self.app.exec()
        self.assertTrue(callback.called)

    def testSingleShotWithCallableInObject(self):
        callback = self.CallbackObject(self.app)
        QTimer.singleShot(100, callback.func)
        self.app.exec()
        self.assertTrue(callback.called)

    def testSingleShotWithCallableInObjectZero(self):
        callback = self.CallbackObject(self.app)
        QTimer.singleShot(0, callback.func)
        self.app.exec()
        self.assertTrue(callback.called)


class SigEmitter(QObject):

    sig1 = Signal()


class TestSingleShotSignal(UsesQApplication):
    '''Test case for QTimer.singleShot connecting to signals'''

    def setUp(self):
        UsesQApplication.setUp(self)
        self.watchdog = WatchDog(self)
        self.called = False

    def tearDown(self):
        del self.watchdog
        del self.called
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        UsesQApplication.tearDown(self)

    def callback(self):
        self.called = True
        self.app.quit()

    def testSingleShotSignal(self):
        emitter = SigEmitter()
        emitter.sig1.connect(self.callback)
        QTimer.singleShot(100, emitter.sig1)
        self.app.exec()
        self.assertTrue(self.called)

    def testSingleShotSignalZero(self):
        emitter = SigEmitter()
        emitter.sig1.connect(self.callback)
        QTimer.singleShot(0, emitter.sig1)
        self.app.exec()
        self.assertTrue(self.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the widgets/dialogs/standarddialogs example from Qt v5.x"""

import sys
from textwrap import dedent

from PySide6.QtCore import QDir, QLibraryInfo, QLocale, QTranslator, Qt, Slot
from PySide6.QtGui import QFont, QPalette
from PySide6.QtWidgets import (QApplication, QColorDialog, QCheckBox, QDialog,
                               QErrorMessage, QFontDialog, QFileDialog, QFrame,
                               QGridLayout, QGroupBox, QInputDialog, QLabel,
                               QLineEdit, QMessageBox, QPushButton,
                               QSizePolicy, QSpacerItem, QToolBox,
                               QVBoxLayout, QWidget)


class DialogOptionsWidget(QGroupBox):
    """Widget displaying a number of check boxes representing the dialog
       options."""
    def __init__(self, zero_value, parent=None):
        super().__init__(parent)
        self._zero_value = zero_value
        self._layout = QVBoxLayout(self)
        self._mapping = {}

    def value(self):
        result = self._zero_value
        for checkbox, value in self._mapping.items():
            if checkbox.isChecked():
                result |= value
        return result

    def add_checkbox(self, text, value):
        checkbox = QCheckBox(text)
        self._layout.addWidget(checkbox)
        self._mapping[checkbox] = value


class Dialog(QDialog):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._open_files_path = ''

        self._error_message_dialog = QErrorMessage(self)

        frame_style = QFrame.Shadow.Sunken | QFrame.Shape.Panel

        self._integer_label = QLabel()
        self._integer_label.setFrameStyle(frame_style)
        self._integer_button = QPushButton("QInputDialog.get&Integer()")

        self._double_label = QLabel()
        self._double_label.setFrameStyle(frame_style)
        self._double_button = QPushButton("QInputDialog.get&Double()")

        self._item_label = QLabel()
        self._item_label.setFrameStyle(frame_style)
        self._item_button = QPushButton("QInputDialog.getIte&m()")

        self._text_label = QLabel()
        self._text_label.setFrameStyle(frame_style)
        self._text_button = QPushButton("QInputDialog.get&Text()")

        self._multiline_text_label = QLabel()
        self._multiline_text_label.setFrameStyle(frame_style)
        self._multiline_text_button = QPushButton("QInputDialog::get&MultiLineText()")

        self._color_label = QLabel()
        self._color_label.setFrameStyle(frame_style)
        self._color_button = QPushButton("QColorDialog.get&Color()")
        self._color_options = DialogOptionsWidget(QColorDialog.ColorDialogOption(0))
        self._color_options.add_checkbox("Show alpha channel",
                                         QColorDialog.ColorDialogOption.ShowAlphaChannel)
        self._color_options.add_checkbox("No buttons",
                                         QColorDialog.ColorDialogOption.NoButtons)

        self._font_label = QLabel()
        self._font_label.setFrameStyle(frame_style)
        self._font_button = QPushButton("QFontDialog.get&Font()")
        self._font_options = DialogOptionsWidget(QFontDialog.FontDialogOption(0))
        self._font_options.add_checkbox("Do not use native dialog",
                                        QFontDialog.FontDialogOption.DontUseNativeDialog)
        self._font_options.add_checkbox("Show scalable fonts",
                                        QFontDialog.FontDialogOption.ScalableFonts)
        self._font_options.add_checkbox("Show non-scalable fonts",
                                        QFontDialog.FontDialogOption.NonScalableFonts)
        self._font_options.add_checkbox("Show monospaced fonts",
                                        QFontDialog.FontDialogOption.MonospacedFonts)
        self._font_options.add_checkbox("Show proportional fonts",
                                        QFontDialog.FontDialogOption.ProportionalFonts)
        self._font_options.add_checkbox("No buttons", QFontDialog.FontDialogOption.NoButtons)

        self._directory_label = QLabel()
        self._directory_label.setFrameStyle(frame_style)
        self._directory_button = QPushButton("QFileDialog.getE&xistingDirectory()")

        self._open_file_name_label = QLabel()
        self._open_file_name_label.setFrameStyle(frame_style)
        self._open_file_name_button = QPushButton("QFileDialog.get&OpenFileName()")

        self._open_file_names_label = QLabel()
        self._open_file_names_label.setFrameStyle(frame_style)
        self._open_file_names_button = QPushButton("QFileDialog.&getOpenFileNames()")

        self._save_file_name_label = QLabel()
        self._save_file_name_label.setFrameStyle(frame_style)
        self._save_file_name_button = QPushButton("QFileDialog.get&SaveFileName()")

        self._file_options = DialogOptionsWidget(QFileDialog.Option(0))
        self._file_options.add_checkbox("Do not use native dialog",
                                        QFileDialog.Option.DontUseNativeDialog)
        self._file_options.add_checkbox("Show directories only",
                                        QFileDialog.Option.ShowDirsOnly)
        self._file_options.add_checkbox("Do not resolve symlinks",
                                        QFileDialog.Option.DontResolveSymlinks)
        self._file_options.add_checkbox("Do not confirm overwrite",
                                        QFileDialog.Option.DontConfirmOverwrite)
        self._file_options.add_checkbox("Readonly", QFileDialog.Option.ReadOnly)
        self._file_options.add_checkbox("Hide name filter details",
                                        QFileDialog.Option.HideNameFilterDetails)
        self._file_options.add_checkbox("Do not use custom directory icons (Windows)",
                                        QFileDialog.Option.DontUseCustomDirectoryIcons)

        self._critical_label = QLabel()
        self._critical_label.setFrameStyle(frame_style)
        self._critical_button = QPushButton("QMessageBox.critica&l()")

        self._information_label = QLabel()
        self._information_label.setFrameStyle(frame_style)
        self._information_button = QPushButton("QMessageBox.i&nformation()")

        self._question_label = QLabel()
        self._question_label.setFrameStyle(frame_style)
        self._question_button = QPushButton("QMessageBox.&question()")

        self._warning_label = QLabel()
        self._warning_label.setFrameStyle(frame_style)
        self._warning_button = QPushButton("QMessageBox.&warning()")

        self._error_button = QPushButton("QErrorMessage.showM&essage()")

        self._integer_button.clicked.connect(self.set_integer)
        self._double_button.clicked.connect(self.set_double)
        self._item_button.clicked.connect(self.set_item)
        self._text_button.clicked.connect(self.set_text)
        self._multiline_text_button.clicked.connect(self.set_multiline_text)

        self._color_button.clicked.connect(self.set_color)
        self._font_button.clicked.connect(self.set_font)
        self._directory_button.clicked.connect(self.set_existing_directory)
        self._open_file_name_button.clicked.connect(self.set_open_file_name)
        self._open_file_names_button.clicked.connect(self.set_open_file_names)
        self._save_file_name_button.clicked.connect(self.set_save_file_name)
        self._critical_button.clicked.connect(self.critical_message)
        self._information_button.clicked.connect(self.information_message)
        self._question_button.clicked.connect(self.question_message)
        self._warning_button.clicked.connect(self.warning_message)
        self._error_button.clicked.connect(self.error_message)

        vertical_layout = QVBoxLayout(self)
        toolbox = QToolBox()

        vertical_layout.addWidget(toolbox)
        page = QWidget()
        layout = QGridLayout(page)
        layout.addWidget(self._integer_button, 0, 0)
        layout.addWidget(self._integer_label, 0, 1)
        layout.addWidget(self._double_button, 1, 0)
        layout.addWidget(self._double_label, 1, 1)
        layout.addWidget(self._item_button, 2, 0)
        layout.addWidget(self._item_label, 2, 1)
        layout.addWidget(self._text_button, 3, 0)
        layout.addWidget(self._text_label, 3, 1)
        layout.addWidget(self._multiline_text_label, 4, 1)
        layout.addWidget(self._multiline_text_button, 4, 0)
        spacer = QSpacerItem(0, 0, QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        layout.addItem(spacer, 5, 0)
        toolbox.addItem(page, "Input Dialogs")

        page = QWidget()
        layout = QGridLayout(page)
        layout.addWidget(self._color_button, 0, 0)
        layout.addWidget(self._color_label, 0, 1)
        spacer = QSpacerItem(0, 0, QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        layout.addItem(spacer, 1, 0)
        layout.addWidget(self._color_options, 2, 0, 1, 2)
        toolbox.addItem(page, "Color Dialog")

        page = QWidget()
        layout = QGridLayout(page)
        layout.addWidget(self._font_button, 0, 0)
        layout.addWidget(self._font_label, 0, 1)
        spacer = QSpacerItem(0, 0, QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        layout.addItem(spacer, 1, 0)
        layout.addWidget(self._font_options, 2, 0, 1, 2)
        toolbox.addItem(page, "Font Dialog")

        page = QWidget()
        layout = QGridLayout(page)
        layout.addWidget(self._directory_button, 0, 0)
        layout.addWidget(self._directory_label, 0, 1)
        layout.addWidget(self._open_file_name_button, 1, 0)
        layout.addWidget(self._open_file_name_label, 1, 1)
        layout.addWidget(self._open_file_names_button, 2, 0)
        layout.addWidget(self._open_file_names_label, 2, 1)
        layout.addWidget(self._save_file_name_button, 3, 0)
        layout.addWidget(self._save_file_name_label, 3, 1)
        spacer = QSpacerItem(0, 0, QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        layout.addItem(spacer, 4, 0)
        layout.addWidget(self._file_options, 5, 0, 1, 2)

        toolbox.addItem(page, "File Dialogs")

        page = QWidget()
        layout = QGridLayout(page)
        layout.addWidget(self._critical_button, 0, 0)
        layout.addWidget(self._critical_label, 0, 1)
        layout.addWidget(self._information_button, 1, 0)
        layout.addWidget(self._information_label, 1, 1)
        layout.addWidget(self._question_button, 2, 0)
        layout.addWidget(self._question_label, 2, 1)
        layout.addWidget(self._warning_button, 3, 0)
        layout.addWidget(self._warning_label, 3, 1)
        layout.addWidget(self._error_button, 4, 0)
        spacer = QSpacerItem(0, 0, QSizePolicy.Policy.Ignored, QSizePolicy.Policy.MinimumExpanding)
        layout.addItem(spacer, 5, 0)
        toolbox.addItem(page, "Message Boxes")

        self.setWindowTitle("Standard Dialogs")

    @Slot()
    def set_integer(self):
        i, ok = QInputDialog.getInt(self,
                                    "QInputDialog.getInteger()", "Percentage:", 25, 0, 100, 1)
        if ok:
            self._integer_label.setText(f"{i}%")

    @Slot()
    def set_double(self):
        d, ok = QInputDialog.getDouble(self, "QInputDialog.getDouble()",
                                       "Amount:", 37.56, -10000, 10000, 2)
        if ok:
            self._double_label.setText(f"${d:g}")

    @Slot()
    def set_item(self):
        items = ("Spring", "Summer", "Fall", "Winter")

        item, ok = QInputDialog.getItem(self, "QInputDialog.getItem()", "Season:", items, 0, False)
        if ok and item:
            self._item_label.setText(item)

    @Slot()
    def set_text(self):
        text, ok = QInputDialog.getText(self, "QInputDialog.getText()",
                                        "User name:", QLineEdit.Normal, QDir.home().dirName())
        if ok and text != '':
            self._text_label.setText(text)

    @Slot()
    def set_multiline_text(self):
        text, ok = QInputDialog.getMultiLineText(self, "QInputDialog::getMultiLineText()",
                                                 "Address:", "John Doe\nFreedom Street")
        if ok and text != '':
            self._multiline_text_label.setText(text)

    @Slot()
    def set_color(self):
        options_value = self._color_options.value()
        options = QColorDialog.ColorDialogOption(options_value)
        color = QColorDialog.getColor(Qt.GlobalColor.green, self, "Select Color", options)

        if color.isValid():
            self._color_label.setText(color.name())
            self._color_label.setPalette(QPalette(color))
            self._color_label.setAutoFillBackground(True)

    @Slot()
    def set_font(self):
        options_value = self._font_options.value()
        options = QFontDialog.FontDialogOptions(options_value)

        default_font = QFont()
        if description := self._font_label.text():
            default_font.fromString(description)

        ok, font = QFontDialog.getFont(default_font, self, "Select Font", options)
        if ok:
            self._font_label.setText(font.key())
            self._font_label.setFont(font)

    @Slot()
    def set_existing_directory(self):
        options_value = self._file_options.value()
        options = QFileDialog.Options(options_value) | QFileDialog.ShowDirsOnly

        directory = QFileDialog.getExistingDirectory(self, "QFileDialog.getExistingDirectory()",
                                                     self._directory_label.text(), options)
        if directory:
            self._directory_label.setText(directory)

    @Slot()
    def set_open_file_name(self):
        options_value = self._file_options.value()
        options = QFileDialog.Options(options_value)

        fileName, _ = QFileDialog.getOpenFileName(self, "QFileDialog.getOpenFileName()",
                                                  self._open_file_name_label.text(),
                                                  "All Files (*);;Text Files (*.txt)", "", options)
        if fileName:
            self._open_file_name_label.setText(fileName)

    @Slot()
    def set_open_file_names(self):
        options_value = self._file_options.value()
        options = QFileDialog.Options(options_value)

        files, _ = QFileDialog.getOpenFileNames(self, "QFileDialog.getOpenFileNames()",
                                                self._open_files_path,
                                                "All Files (*);;Text Files (*.txt)", "", options)
        if files:
            self._open_files_path = files[0]
            file_list = ', '.join(files)
            self._open_file_names_label.setText(f"[{file_list}]")

    @Slot()
    def set_save_file_name(self):
        options_value = self._file_options.value()
        options = QFileDialog.Options(options_value)

        fileName, _ = QFileDialog.getSaveFileName(self, "QFileDialog.getSaveFileName()",
                                                  self._save_file_name_label.text(),
                                                  "All Files (*);;Text Files (*.txt)", "", options)
        if fileName:
            self._save_file_name_label.setText(fileName)

    @Slot()
    def critical_message(self):
        m = dedent("""\
                   Activating the liquid oxygen stirring fans caused an
                   explosion in one of the tanks. Liquid oxygen levels
                   are getting low. This may jeopardize the moon landing mission.""")
        msg_box = QMessageBox(QMessageBox.Icon.Critical, "QMessageBox.critical()",
                              "Houston, we have a problem",
                              QMessageBox.StandardButton.Abort | QMessageBox.StandardButton.Retry
                              | QMessageBox.StandardButton.Ignore, self)
        msg_box.setInformativeText(m)
        reply = msg_box.exec()
        if reply == QMessageBox.StandardButton.Abort:
            self._critical_label.setText("Abort")
        elif reply == QMessageBox.StandardButton.Retry:
            self._critical_label.setText("Retry")
        else:
            self._critical_label.setText("Ignore")

    @Slot()
    def information_message(self):
        m = dedent("""\
                   This phrase was often used by public address announcers at
                   the conclusion of Elvis Presley concerts in order to
                   disperse audiences who lingered in hopes of an encore.
                   It has since become a catchphrase and punchline.""")
        msg_box = QMessageBox(QMessageBox.Icon.Information, "QMessageBox.information()",
                              "Elvis has left the building.",
                              QMessageBox.StandardButton.Ok, self)
        msg_box.setInformativeText(m)
        reply = msg_box.exec()
        if reply == QMessageBox.StandardButton.Ok:
            self._information_label.setText("OK")
        else:
            self._information_label.setText("Escape")

    @Slot()
    def question_message(self):
        m = dedent("""\
                   A cheeseburger is a hamburger topped with cheese.
                   Traditionally, the slice of cheese is placed on top of the
                   meat patty. The cheese is usually added to the cooking
                   hamburger patty shortly before serving, which allows the
                   cheese to melt.""")
        msg_box = QMessageBox(QMessageBox.Icon.Question, "QMessageBox.question()",
                              "Would you like cheese with that?",
                              QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
                              | QMessageBox.StandardButton.Cancel)
        msg_box.setInformativeText(m)
        reply = msg_box.exec()
        if reply == QMessageBox.StandardButton.Yes:
            self._question_label.setText("Yes")
        elif reply == QMessageBox.StandardButton.No:
            self._question_label.setText("No")
        else:
            self._question_label.setText("Cancel")

    @Slot()
    def warning_message(self):
        msg_box = QMessageBox(QMessageBox.Icon.Warning, "QMessageBox.warning()",
                              "Delete the only copy of your movie manuscript?",
                              QMessageBox.StandardButton.NoButton, self)
        m = "You've been working on this manuscript for 738 days now. Hang in there!"
        msg_box.setInformativeText(m)
        msg_box.setDetailedText('"A long time ago in a galaxy far, far away...."')
        msg_box.addButton("&Keep", QMessageBox.ButtonRole.AcceptRole)
        msg_box.addButton("Delete", QMessageBox.ButtonRole.RejectRole)
        if msg_box.exec() == QMessageBox.ButtonRole.AcceptRole:
            self._warning_label.setText("Keep")
        else:
            self._warning_label.setText("Delete")

    @Slot()
    def error_message(self):
        m = dedent("""\
                   This dialog shows and remembers error messages. If the
                   user chooses to not  show the dialog again, the dialog
                   will not appear again if QErrorMessage.showMessage()
                   is called with the same message.""")
        self._error_message_dialog.showMessage(m)
        m = dedent("""\
                   You can queue up error messages, and they will be
                   shown one after each other. Each message maintains
                   its own state for whether it will be shown again
                   the next time QErrorMessage::showMessage() is called
                   with the same message.""")
        self._error_message_dialog.showMessage(m)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    translator = QTranslator(app)
    if translator.load(QLocale.system(), "qtbase", "_",
                       QLibraryInfo.path(QLibraryInfo.LibraryPath.TranslationsPath)):
        app.installTranslator(translator)
    dialog = Dialog()
    availableGeometry = dialog.screen().availableGeometry()
    dialog.resize(availableGeometry.width() / 3, availableGeometry.height() * 2 / 3)
    dialog.move((availableGeometry.width() - dialog.width()) / 2,
                (availableGeometry.height() - dialog.height()) / 2)
    sys.exit(dialog.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import logging
import os
import sys
from argparse import ArgumentParser, RawTextHelpFormatter
from pathlib import Path

sys.path.append(os.fspath(Path(__file__).parent / "snippets_translate"))

from converter import snippet_translate

DESCRIPTION = "Tool to convert C++ to Python based on snippets_translate"


def create_arg_parser(desc):
    parser = ArgumentParser(description=desc,
                            formatter_class=RawTextHelpFormatter)
    parser.add_argument("--stdout", "-s", action="store_true",
                        help="Write to stdout")
    parser.add_argument("--force", "-f", action="store_true",
                        help="Force overwrite of existing files")
    parser.add_argument("files", type=str, nargs="+",
                        help="C++ source file(s)")
    return parser


if __name__ == "__main__":
    arg_parser = create_arg_parser(DESCRIPTION)
    args = arg_parser.parse_args()
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger(__name__)

    for input_file_str in args.files:
        input_file = Path(input_file_str)
        if not input_file.is_file():
            logger.error(f"{input_file_str} does not exist or is not a file.")
            sys.exit(-1)

        if input_file.suffix != ".cpp" and input_file.suffix != ".h":
            logger.error(f"{input_file} does not appear to be a C++ file.")
            sys.exit(-1)

        translated_lines = [f"# Converted from {input_file.name}\n"]
        for line in input_file.read_text().split("\n"):
            translated_lines.append(snippet_translate(line))
        translated = "\n".join(translated_lines)

        if args.stdout:
            sys.stdout.write(translated)
        else:
            target_file = input_file.parent / (input_file.stem + ".py")
            if target_file.exists():
                if not target_file.is_file():
                    logger.error(f"{target_file} exists and is not a file.")
                    sys.exit(-1)
                if not args.force:
                    logger.error(f"{target_file} exists. Use -f to overwrite.")
                    sys.exit(-1)

            target_file.write_text(translated)
            logger.info(f"Wrote {target_file}.")



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Tests for deleting a child object in python'''

import gc
import os
import random
import string
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType


class DeleteChildInPython(unittest.TestCase):
    '''Test case for deleting (unref) a child in python'''

    def testDeleteChild(self):
        '''Delete child in python should not invalidate child'''
        parent = ObjectType()
        child = ObjectType(parent)
        name = ''.join(random.sample(string.ascii_letters, 5))
        child.setObjectName(name)

        del child
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        new_child = parent.children()[0]
        self.assertEqual(new_child.objectName(), name)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for passing invalid callbacks to QObject.connect'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


class InvalidCallback(unittest.TestCase):
    '''Test case for passing an invalid callback to QObject.connect'''

    def setUp(self):
        # Acquire resources
        self.obj = QObject()

    def tearDown(self):
        # Release resources
        try:
            del self.obj
        except AttributeError:
            pass
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testIntegerCb(self):
        # Test passing an int as callback to QObject.connect
        self.assertRaises(TypeError, self.obj.destroyed.connect, 42)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QTimer

from helper.usesqapplication import UsesQApplication


class TestBugPYSIDE2745(UsesQApplication):

    def setUp(self):
        UsesQApplication.setUp(self)
        self.counter = 0

    def fail(self):
        self.counter += 1
        raise Exception()

    def test_fail(self):
        QTimer.singleShot(0, self.fail)
        QTimer.singleShot(0, self.fail)
        QTimer.singleShot(1, self.app.quit)
        self.app.exec()
        self.assertEqual(self.counter, 2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 367: http://bugs.openbossa.org/show_bug.cgi?id=367'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtGui import QStandardItem, QStandardItemModel


class BugTest(UsesQApplication):
    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testCase(self):
        model = QStandardItemModel()
        parentItem = model.invisibleRootItem()
        for i in range(10):
            item = QStandardItem()
            rcount = sys.getrefcount(item)
            parentItem.appendRow(item)
            self.assertEqual(rcount + 1, sys.getrefcount(item))
            parentItem = item

        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit tests for QObject's tr static methods.'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject

#from helper.usesqapplication import UsesQApplication


class QObjectTrTest(unittest.TestCase):
    '''Test case to check if QObject tr static methods could be treated as instance methods.'''

    def setUp(self):
        self.obj = QObject()

    def tearDown(self):
        del self.obj
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testTrCommonCase(self):
        # Test common case for QObject.tr
        invar1 = 'test1'
        outvar1 = self.obj.tr(invar1)
        invar2 = 'test2'
        outvar2 = self.obj.tr(invar2, 'test comment')
        self.assertEqual((invar1, invar2), (outvar1, outvar2))

    def testTrAsInstanceMethod(self):
        # Test QObject.tr as instance.
        # PYSIDE-1252: This works now as a class method!
        invar1 = 'test1'
        outvar1 = QObject.tr(invar1)
        invar2 = 'test2'
        outvar2 = QObject.tr(invar2, 'test comment')
        self.assertEqual((invar1, invar2), (outvar1, outvar2))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QDataStream, QByteArray, QIODevice, Qt
from PySide6.QtGui import QPixmap, QColor

from helper.usesqapplication import UsesQApplication


class QPixmapQDatastream(UsesQApplication):
    '''QDataStream <<>> QPixmap'''

    def setUp(self):
        super(QPixmapQDatastream, self).setUp()
        self.source_pixmap = QPixmap(100, 100)
        # PYSIDE-1533: Use Qt.transparent to force Format_ARGB32_Premultiplied
        # when converting to QImage in any case.
        self.source_pixmap.fill(Qt.GlobalColor.transparent)
        self.output_pixmap = QPixmap()
        self.buffer = QByteArray()
        self.read_stream = QDataStream(self.buffer, QIODevice.OpenModeFlag.ReadOnly)
        self.write_stream = QDataStream(self.buffer, QIODevice.OpenModeFlag.WriteOnly)

    def testStream(self):
        self.write_stream << self.source_pixmap

        self.read_stream >> self.output_pixmap

        image = self.output_pixmap.toImage()
        pixel = image.pixel(10, 10)
        self.assertEqual(pixel, QColor(Qt.GlobalColor.transparent).rgba())
        self.assertEqual(self.source_pixmap.toImage(), image)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Signal, Slot
from PySide6.QtWidgets import QMainWindow, QPushButton, QTextEdit
from helper.usesqapplication import UsesQApplication


class MyWindow(QMainWindow):
    appendText = Signal(str)

    @Slot()
    def onButtonPressed(self):
        self.appendText.emit("PySide")

    def __init__(self, parent=None):
        super().__init__(parent)

        self.textEdit = QTextEdit()
        self.btn = QPushButton("ClickMe")
        self.btn.clicked.connect(self.onButtonPressed)
        self.appendText.connect(self.textEdit.append)

    def start(self):
        self.btn.click()

    def text(self):
        return self.textEdit.toPlainText()


class testSignalWithCPPSlot(UsesQApplication):

    def testEmission(self):
        w = MyWindow()
        w.start()
        self.assertEqual(w.text(), "PySide")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import math
from pathlib import Path

from PySide6.QtCore import QObject, Qt, Slot
from PySide6.QtDataVisualization import (Q3DTheme, QAbstract3DGraph,
                                         QHeightMapSurfaceDataProxy,
                                         QSurface3DSeries, QSurfaceDataItem,
                                         QSurfaceDataProxy, QValue3DAxis)
from PySide6.QtGui import QImage, QLinearGradient, QVector3D
from PySide6.QtWidgets import QSlider

SAMPLE_COUNT_X = 50
SAMPLE_COUNT_Z = 50
HEIGHT_MAP_GRID_STEP_X = 6
HEIGHT_MAP_GRID_STEP_Z = 6
SAMPLE_MIN = -8.0
SAMPLE_MAX = 8.0


class SurfaceGraph(QObject):
    def __init__(self, surface, parent=None):
        super().__init__(parent)

        self._graph = surface
        self._graph.setAxisX(QValue3DAxis())
        self._graph.setAxisY(QValue3DAxis())
        self._graph.setAxisZ(QValue3DAxis())

        self._sqrtSinProxy = QSurfaceDataProxy()
        self._sqrtSinSeries = QSurface3DSeries(self._sqrtSinProxy)
        self.fill_sqrt_sin_proxy()

        image_file = Path(__file__).parent / "mountain.png"
        height_map_image = QImage(image_file)
        self._heightMapProxy = QHeightMapSurfaceDataProxy(height_map_image)
        self._heightMapSeries = QSurface3DSeries(self._heightMapProxy)
        self._heightMapSeries.setItemLabelFormat("(@xLabel, @zLabel): @yLabel")
        self._heightMapProxy.setValueRanges(34.0, 40.0, 18.0, 24.0)

        self._heightMapWidth = height_map_image.width()
        self._heightMapHeight = height_map_image.height()

        self._axisMinSliderX = QSlider()
        self._axisMaxSliderX = QSlider()
        self._axisMinSliderZ = QSlider()
        self._axisMaxSliderZ = QSlider()
        self._rangeMinX = 0.0
        self._rangeMinZ = 0.0
        self._stepX = 0.0
        self._stepZ = 0.0

    def fill_sqrt_sin_proxy(self):
        step_x = (SAMPLE_MAX - SAMPLE_MIN) / float(SAMPLE_COUNT_X - 1)
        step_z = (SAMPLE_MAX - SAMPLE_MIN) / float(SAMPLE_COUNT_Z - 1)

        data_array = []
        for i in range(SAMPLE_COUNT_Z):
            new_row = []
            # Keep values within range bounds, since just adding step can cause
            # minor drift due to the rounding errors.
            z = min(SAMPLE_MAX, (i * step_z + SAMPLE_MIN))
            for j in range(SAMPLE_COUNT_X):
                x = min(SAMPLE_MAX, (j * step_x + SAMPLE_MIN))
                R = math.sqrt(z * z + x * x) + 0.01
                y = (math.sin(R) / R + 0.24) * 1.61
                new_row.append(QSurfaceDataItem(QVector3D(x, y, z)))
            data_array.append(new_row)

        self._sqrtSinProxy.resetArray(data_array)

    @Slot(bool)
    def enable_sqrt_sin_model(self, enable):
        if enable:
            self._sqrtSinSeries.setDrawMode(QSurface3DSeries.DrawSurfaceAndWireframe)
            self._sqrtSinSeries.setFlatShadingEnabled(True)

            self._graph.axisX().setLabelFormat("%.2f")
            self._graph.axisZ().setLabelFormat("%.2f")
            self._graph.axisX().setRange(SAMPLE_MIN, SAMPLE_MAX)
            self._graph.axisY().setRange(0.0, 2.0)
            self._graph.axisZ().setRange(SAMPLE_MIN, SAMPLE_MAX)
            self._graph.axisX().setLabelAutoRotation(30)
            self._graph.axisY().setLabelAutoRotation(90)
            self._graph.axisZ().setLabelAutoRotation(30)

            self._graph.removeSeries(self._heightMapSeries)
            self._graph.addSeries(self._sqrtSinSeries)

            # Reset range sliders for Sqrt&Sin
            self._rangeMinX = SAMPLE_MIN
            self._rangeMinZ = SAMPLE_MIN
            self._stepX = (SAMPLE_MAX - SAMPLE_MIN) / float(SAMPLE_COUNT_X - 1)
            self._stepZ = (SAMPLE_MAX - SAMPLE_MIN) / float(SAMPLE_COUNT_Z - 1)
            self._axisMinSliderX.setMaximum(SAMPLE_COUNT_X - 2)
            self._axisMinSliderX.setValue(0)
            self._axisMaxSliderX.setMaximum(SAMPLE_COUNT_X - 1)
            self._axisMaxSliderX.setValue(SAMPLE_COUNT_X - 1)
            self._axisMinSliderZ.setMaximum(SAMPLE_COUNT_Z - 2)
            self._axisMinSliderZ.setValue(0)
            self._axisMaxSliderZ.setMaximum(SAMPLE_COUNT_Z - 1)
            self._axisMaxSliderZ.setValue(SAMPLE_COUNT_Z - 1)

    @Slot(bool)
    def enable_height_map_model(self, enable):
        if enable:
            self._heightMapSeries.setDrawMode(QSurface3DSeries.DrawSurface)
            self._heightMapSeries.setFlatShadingEnabled(False)

            self._graph.axisX().setLabelFormat("%.1f N")
            self._graph.axisZ().setLabelFormat("%.1f E")
            self._graph.axisX().setRange(34.0, 40.0)
            self._graph.axisY().setAutoAdjustRange(True)
            self._graph.axisZ().setRange(18.0, 24.0)

            self._graph.axisX().setTitle("Latitude")
            self._graph.axisY().setTitle("Height")
            self._graph.axisZ().setTitle("Longitude")

            self._graph.removeSeries(self._sqrtSinSeries)
            self._graph.addSeries(self._heightMapSeries)

            # Reset range sliders for height map
            map_grid_count_x = self._heightMapWidth / HEIGHT_MAP_GRID_STEP_X
            map_grid_count_z = self._heightMapHeight / HEIGHT_MAP_GRID_STEP_Z
            self._rangeMinX = 34.0
            self._rangeMinZ = 18.0
            self._stepX = 6.0 / float(map_grid_count_x - 1)
            self._stepZ = 6.0 / float(map_grid_count_z - 1)
            self._axisMinSliderX.setMaximum(map_grid_count_x - 2)
            self._axisMinSliderX.setValue(0)
            self._axisMaxSliderX.setMaximum(map_grid_count_x - 1)
            self._axisMaxSliderX.setValue(map_grid_count_x - 1)
            self._axisMinSliderZ.setMaximum(map_grid_count_z - 2)
            self._axisMinSliderZ.setValue(0)
            self._axisMaxSliderZ.setMaximum(map_grid_count_z - 1)
            self._axisMaxSliderZ.setValue(map_grid_count_z - 1)

    @Slot(int)
    def adjust_xmin(self, minimum):
        min_x = self._stepX * float(minimum) + self._rangeMinX

        maximum = self._axisMaxSliderX.value()
        if minimum >= maximum:
            maximum = minimum + 1
            self._axisMaxSliderX.setValue(maximum)
        max_x = self._stepX * maximum + self._rangeMinX

        self.set_axis_xrange(min_x, max_x)

    @Slot(int)
    def adjust_xmax(self, maximum):
        max_x = self._stepX * float(maximum) + self._rangeMinX

        minimum = self._axisMinSliderX.value()
        if maximum <= minimum:
            minimum = maximum - 1
            self._axisMinSliderX.setValue(minimum)
        min_x = self._stepX * minimum + self._rangeMinX

        self.set_axis_xrange(min_x, max_x)

    @Slot(int)
    def adjust_zmin(self, minimum):
        min_z = self._stepZ * float(minimum) + self._rangeMinZ

        maximum = self._axisMaxSliderZ.value()
        if minimum >= maximum:
            maximum = minimum + 1
            self._axisMaxSliderZ.setValue(maximum)
        max_z = self._stepZ * maximum + self._rangeMinZ

        self.set_axis_zrange(min_z, max_z)

    @Slot(int)
    def adjust_zmax(self, maximum):
        max_x = self._stepZ * float(maximum) + self._rangeMinZ

        minimum = self._axisMinSliderZ.value()
        if maximum <= minimum:
            minimum = maximum - 1
            self._axisMinSliderZ.setValue(minimum)
        min_x = self._stepZ * minimum + self._rangeMinZ

        self.set_axis_zrange(min_x, max_x)

    def set_axis_xrange(self, minimum, maximum):
        self._graph.axisX().setRange(minimum, maximum)

    def set_axis_zrange(self, minimum, maximum):
        self._graph.axisZ().setRange(minimum, maximum)

    @Slot(int)
    def change_theme(self, theme):
        self._graph.activeTheme().setType(Q3DTheme.Theme(theme))

    @Slot()
    def set_black_to_yellow_gradient(self):
        gr = QLinearGradient()
        gr.setColorAt(0.0, Qt.black)
        gr.setColorAt(0.33, Qt.blue)
        gr.setColorAt(0.67, Qt.red)
        gr.setColorAt(1.0, Qt.yellow)

        series = self._graph.seriesList()[0]
        series.setBaseGradient(gr)
        series.setColorStyle(Q3DTheme.ColorStyleRangeGradient)

    @Slot()
    def set_green_to_red_gradient(self):
        gr = QLinearGradient()
        gr.setColorAt(0.0, Qt.darkGreen)
        gr.setColorAt(0.5, Qt.yellow)
        gr.setColorAt(0.8, Qt.red)
        gr.setColorAt(1.0, Qt.darkRed)

        series = self._graph.seriesList()[0]
        series.setBaseGradient(gr)
        series.setColorStyle(Q3DTheme.ColorStyleRangeGradient)

    @Slot()
    def toggle_mode_none(self):
        self._graph.setSelectionMode(QAbstract3DGraph.SelectionNone)

    @Slot()
    def toggle_mode_item(self):
        self._graph.setSelectionMode(QAbstract3DGraph.SelectionItem)

    @Slot()
    def toggle_mode_slice_row(self):
        self._graph.setSelectionMode(
            QAbstract3DGraph.SelectionItemAndRow | QAbstract3DGraph.SelectionSlice
        )

    @Slot()
    def toggle_mode_slice_column(self):
        self._graph.setSelectionMode(
            QAbstract3DGraph.SelectionItemAndColumn | QAbstract3DGraph.SelectionSlice
        )

    def set_axis_min_slider_x(self, slider):
        self._axisMinSliderX = slider

    def set_axis_max_slider_x(self, slider):
        self._axisMaxSliderX = slider

    def set_axis_min_slider_z(self, slider):
        self._axisMinSliderZ = slider

    def set_axis_max_slider_z(self, slider):
        self._axisMaxSliderZ = slider



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Simple test case for accessing the exposed C++ class fields.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Derived, Point, ObjectType


REF_COUNT_DELTA = 2 if sys.version_info >= (3, 14) else 1


class TestAccessingCppFields(unittest.TestCase):
    '''Simple test case for accessing the exposed C++ class fields.'''

    def testAccessingPrimitiveTypeField(self):
        '''Reads and writes a primitive type (in this case an 'int') field.'''
        d = Derived()
        self.assertEqual(type(d.primitiveField), int)

        # attribution
        old_value = d.primitiveField
        new_value = 2255
        d.primitiveField = new_value
        self.assertEqual(d.primitiveField, new_value)
        self.assertNotEqual(d.primitiveField, old_value)

        # attribution with a convertible type
        value = 1.2
        d.primitiveField = value
        self.assertEqual(d.primitiveField, int(value))

        # attribution with invalid type
        self.assertRaises(TypeError, lambda: setattr(d, 'primitiveField', None))

    def testAccessingRenamedFields(self):
        '''Reads and writes a renamed field.'''
        d = Derived()
        self.assertEqual(type(d.renamedField), int)
        old_value = d.renamedField
        new_value = 2255
        d.renamedField = new_value
        self.assertEqual(d.renamedField, new_value)
        self.assertNotEqual(d.renamedField, old_value)

    def testAccessingReadOnlyFields(self):
        '''Tests a read-only field.'''
        d = Derived()
        self.assertEqual(type(d.readOnlyField), int)
        old_value = d.readOnlyField
        try:
            d.readOnlyField = 25555
        except AttributeError:
            pass
        self.assertEqual(d.readOnlyField, old_value)

    def testAccessingUsersPrimitiveTypeField(self):
        '''Reads and writes an user's primitive type (in this case an 'Complex') field.'''
        d = Derived()
        self.assertEqual(type(d.userPrimitiveField), complex)

        # attribution
        old_value = d.userPrimitiveField
        new_value = complex(1.1, 2.2)
        d.userPrimitiveField = new_value
        self.assertEqual(d.userPrimitiveField, new_value)
        self.assertNotEqual(d.userPrimitiveField, old_value)

        # attribution with invalid type
        self.assertRaises(TypeError, lambda: setattr(d, 'userPrimitiveField', None))

    def testAccessingValueTypeField(self):
        '''Reads and writes a value type (in this case a 'Point') field.'''
        d = Derived()
        self.assertEqual(type(d.valueTypeField), Point)

        # attribution
        old_value = d.valueTypeField  # noqa: F841
        new_value = Point(-10, 537)
        d.valueTypeField = new_value
        self.assertEqual(d.valueTypeField, new_value)

        #object modify
        d.valueTypeField.setX(10)
        d.valueTypeField.setY(20)
        self.assertEqual(d.valueTypeField.x(), 10)
        self.assertEqual(d.valueTypeField.y(), 20)

        # attribution with invalid type
        self.assertRaises(TypeError, lambda: setattr(d, 'valueTypeField', 123))

    def testAccessingObjectTypeField(self):
        '''Reads and writes a object type (in this case an 'ObjectType') field.'''
        d = Derived()

        # attribution
        old_value = d.objectTypeField
        new_value = ObjectType()
        d.objectTypeField = new_value
        self.assertEqual(d.objectTypeField, new_value)
        self.assertNotEqual(d.objectTypeField, old_value)

        # attribution with a convertible type
        value = None
        d.objectTypeField = value
        self.assertEqual(d.objectTypeField, value)

        # attribution with invalid type
        self.assertRaises(TypeError, lambda: setattr(d, 'objectTypeField', 123))

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountingAccessingObjectTypeField(self):
        '''Accessing a object type field should respect the reference counting rules.'''
        d = Derived()

        # attributing object to instance's field should increase its reference count
        o1 = ObjectType()
        refcount1 = sys.getrefcount(o1)
        d.objectTypeField = o1
        self.assertEqual(d.objectTypeField, o1)
        self.assertEqual(sys.getrefcount(d.objectTypeField), refcount1 + REF_COUNT_DELTA)

        # attributing a new object to instance's field should decrease the previous
        # object's reference count
        o2 = ObjectType()
        refcount2 = sys.getrefcount(o2)
        d.objectTypeField = o2
        self.assertEqual(d.objectTypeField, o2)
        self.assertEqual(sys.getrefcount(o1), refcount1)
        self.assertEqual(sys.getrefcount(d.objectTypeField), refcount2 + REF_COUNT_DELTA)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testRefCountingOfReferredObjectAfterDeletingReferrer(self):
        '''Deleting the object referring to other object should decrease the
        reference count of the referee.'''
        d = Derived()
        o = ObjectType()
        refcount = sys.getrefcount(o)
        d.objectTypeField = o
        self.assertEqual(sys.getrefcount(o), refcount + 1)
        del d
        self.assertEqual(sys.getrefcount(o), refcount)

    def testStaticField(self):
        self.assertEqual(Derived.staticPrimitiveField, 0)

    def testAccessingUnsignedIntBitField(self):
        d = Derived()

        # attribution
        old_value = d.bitField
        new_value = 1
        d.bitField = new_value
        self.assertEqual(d.bitField, new_value)
        self.assertNotEqual(d.bitField, old_value)

        # attribution with a convertible type
        value = 1.2
        d.bitField = value
        self.assertEqual(d.bitField, int(value))

        # attribution with invalid type
        self.assertRaises(TypeError, lambda: setattr(d, 'bitField', None))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QTabWidget


class TestBug988 (unittest.TestCase):

    def callback(self, arg):
        self.arg = arg

    def testIt(self):
        self.arg = None
        app = QApplication([])  # noqa: F841
        obj = QTabWidget()
        obj.currentChanged.connect(self.callback)
        obj.currentChanged.emit(5)
        self.assertEqual(self.arg, 5)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import numpy
from pathlib import Path
import weakref
from OpenGL.GL import (GL_TEXTURE_MAG_FILTER, GL_TEXTURE_MIN_FILTER,
                       GL_NEAREST, GL_RGBA, GL_TEXTURE_2D, GL_UNSIGNED_BYTE)

from PySide6.QtGui import (QMouseEvent, QOffscreenSurface,
                           QOpenGLContext, QSurface,
                           QSurfaceFormat, QWindow)
from PySide6.QtOpenGL import QOpenGLFramebufferObject
from PySide6.QtQml import QQmlComponent, QQmlEngine
from PySide6.QtQuick import (QQuickGraphicsDevice,
                             QQuickRenderControl,
                             QQuickRenderTarget, QQuickWindow)
from PySide6.QtCore import QCoreApplication, QTimer, QUrl, Slot
from shiboken6 import VoidPtr

from cuberenderer import CubeRenderer


class RenderControl(QQuickRenderControl):
    def __init__(self, window=None):
        super().__init__()
        self._window = window

    def renderWindow(self, offset):
        return self._window()  # Dereference the weak reference


class WindowSingleThreaded(QWindow):

    def __init__(self):
        super().__init__()
        self.m_rootItem = None
        self.m_device = None
        self.m_texture_ids = numpy.array([0], dtype=numpy.uint32)

        self.m_quickInitialized = False
        self.m_quickReady = False
        self.m_dpr = 0
        self.m_status_conn_id = None
        self.setSurfaceType(QSurface.SurfaceType.OpenGLSurface)

        format = QSurfaceFormat()
        # Qt Quick may need a depth and stencil buffer. Always make sure these
        # are available.
        format.setDepthBufferSize(16)
        format.setStencilBufferSize(8)
        self.setFormat(format)

        self.m_context = QOpenGLContext()
        self.m_context.setFormat(format)
        self.m_context.create()

        self.m_offscreenSurface = QOffscreenSurface()
        # Pass m_context.format(), not format. Format does not specify and
        # color buffer sizes, while the context, that has just been created,
        # reports a format that has these values filled in. Pass self to the
        # offscreen surface to make sure it will be compatible with the
        # context's configuration.
        self.m_offscreenSurface.setFormat(self.m_context.format())
        self.m_offscreenSurface.create()

        self.m_cubeRenderer = CubeRenderer(self.m_offscreenSurface)

        self.m_renderControl = RenderControl(weakref.ref(self))

        # Create a QQuickWindow that is associated with out render control.
        # Note that this window never gets created or shown, meaning that
        # will never get an underlying native (platform) window.
        self.m_quickWindow = QQuickWindow(self.m_renderControl)

        # Create a QML engine.
        self.m_qmlEngine = QQmlEngine()
        if not self.m_qmlEngine.incubationController():
            c = self.m_quickWindow.incubationController()
            self.m_qmlEngine.setIncubationController(c)

        # When Quick says there is a need to render, we will not render
        # immediately. Instead, a timer with a small interval is used
        # to get better performance.
        self.m_updateTimer = QTimer()
        self.m_updateTimer.setSingleShot(True)
        self.m_updateTimer.setInterval(5)
        self.m_updateTimer.timeout.connect(self.render)

        # Now hook up the signals. For simplicy we don't differentiate between
        # renderRequested (only render is needed, no sync) and sceneChanged
        # (polish and sync is needed too).
        self.m_quickWindow.sceneGraphInitialized.connect(self.createTexture)
        self.m_quickWindow.sceneGraphInvalidated.connect(self.destroyTexture)
        self.m_renderControl.renderRequested.connect(self.requestUpdate)
        self.m_renderControl.sceneChanged.connect(self.requestUpdate)

        # Just recreating the texture on resize is not sufficient, when moving
        # between screens with different devicePixelRatio the QWindow size may
        # remain the same but the texture dimension is to change regardless.
        self.screenChanged.connect(self.handleScreenChange)

    def __del__(self):
        # Make sure the context is current while doing cleanup. Note that
        # we use the offscreen surface here because passing 'self' at self
        # point is not safe: the underlying platform window may already be
        # destroyed. To avoid all the trouble, use another surface that is
        # valid for sure.
        self.m_context.makeCurrent(self.m_offscreenSurface)

        del self.m_qmlComponent
        del self.m_qmlEngine
        del self.m_quickWindow
        del self.m_renderControl

        if self.texture_id():
            self.m_context.functions().glDeleteTextures(1, self.m_texture_ids)

        self.m_context.doneCurrent()

    def texture_id(self):
        return self.m_texture_ids[0]

    def set_texture_id(self, texture_id):
        self.m_texture_ids[0] = texture_id

    @Slot()
    def createTexture(self):
        # The scene graph has been initialized. It is now time to create a
        # texture and associate it with the QQuickWindow.
        self.m_dpr = self.devicePixelRatio()
        self.m_textureSize = self.size() * self.m_dpr
        f = self.m_context.functions()
        f.glGenTextures(1, self.m_texture_ids)
        f.glBindTexture(GL_TEXTURE_2D, self.texture_id())

        f.glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        f.glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        null = VoidPtr(0)
        f.glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.m_textureSize.width(),
                       self.m_textureSize.height(), 0,
                       GL_RGBA, GL_UNSIGNED_BYTE, null)
        target = QQuickRenderTarget.fromOpenGLTexture(self.texture_id(),
                                                      self.m_textureSize)
        self.m_quickWindow.setRenderTarget(target)

    @Slot()
    def destroyTexture(self):
        self.m_context.functions().glDeleteTextures(1, self.m_texture_ids)
        self.set_texture_id(0)

    @Slot()
    def render(self):
        if not self.m_context.makeCurrent(self.m_offscreenSurface):
            return

        # Polish, synchronize and render the next frame (into our texture).
        # In this example everything happens on the same thread and therefore
        # all three steps are performed in succession from here. In a threaded
        # setup the render() call would happen on a separate thread.
        self.m_renderControl.beginFrame()
        self.m_renderControl.polishItems()
        self.m_renderControl.sync()
        self.m_renderControl.render()
        self.m_renderControl.endFrame()

        QOpenGLFramebufferObject.bindDefault()
        self.m_context.functions().glFlush()

        self.m_quickReady = True

        # Get something onto the screen.
        texture_id = self.texture_id() if self.m_quickReady else 0
        self.m_cubeRenderer.render(self, self.m_context, texture_id)

    def requestUpdate(self):
        if not self.m_updateTimer.isActive():
            self.m_updateTimer.start()

    def run(self):
        if self.m_status_conn_id:
            self.m_qmlComponent.statusChanged.disconnect(self.m_status_conn_id)
            self.m_status_conn_id = None

        if self.m_qmlComponent.isError():
            for error in self.m_qmlComponent.errors():
                print(error.url().toString(), error.line(), error.toString())
            return

        self.m_rootItem = self.m_qmlComponent.create()
        if self.m_qmlComponent.isError():
            for error in self.m_qmlComponent.errors():
                print(error.url().toString(), error.line(), error.toString())
            return

        if not self.m_rootItem:
            print("run: Not a QQuickItem")
            del self.m_rootItem

        # The root item is ready. Associate it with the window.
        self.m_rootItem.setParentItem(self.m_quickWindow.contentItem())

        # Update item and rendering related geometries.
        self.updateSizes()

        # Initialize the render control and our OpenGL resources.
        self.m_context.makeCurrent(self.m_offscreenSurface)
        self.m_device = QQuickGraphicsDevice.fromOpenGLContext(self.m_context)
        self.m_quickWindow.setGraphicsDevice(self.m_device)
        self.m_renderControl.initialize()
        self.m_quickInitialized = True

    def updateSizes(self):
        # Behave like SizeRootObjectToView.
        w = self.width()
        h = self.height()
        self.m_rootItem.setWidth(w)
        self.m_rootItem.setHeight(h)
        self.m_quickWindow.setGeometry(0, 0, w, h)
        self.m_cubeRenderer.resize(w, h)

    def startQuick(self, filename):
        url = QUrl.fromLocalFile(filename)
        self.m_qmlComponent = QQmlComponent(self.m_qmlEngine, url)
        if self.m_qmlComponent.isLoading():
            self.m_status_conn_id = self.m_qmlComponent.statusChanged.connect(self.run)
        else:
            self.run()

    def exposeEvent(self, event):
        if self.isExposed() and not self.m_quickInitialized:
            texture_id = self.texture_id() if self.m_quickReady else 0
            self.m_cubeRenderer.render(self, self.m_context, texture_id)
            qml_file = Path(__file__).parent / "demo.qml"
            self.startQuick(qml_file)

    def resizeTexture(self):
        if self.m_rootItem and self.m_context.makeCurrent(self.m_offscreenSurface):
            self.m_context.functions().glDeleteTextures(1, self.m_texture_ids)
            self.set_texture_id(0)
            self.createTexture()
            self.m_context.doneCurrent()
            self.updateSizes()
            self.render()

    def resizeEvent(self, event):
        # If self is a resize after the scene is up and running, recreate the
        # texture and the Quick item and scene.
        if (self.texture_id()
                and self.m_textureSize != self.size() * self.devicePixelRatio()):
            self.resizeTexture()

    @Slot()
    def handleScreenChange(self):
        if self.m_dpr != self.devicePixelRatio():
            self.resizeTexture()

    def mousePressEvent(self, e):
        # Use the constructor taking position and globalPosition. That puts
        # position into the event's position and scenePosition, and
        # globalPosition into the event's globalPosition. This way the
        # scenePosition in `e` is ignored and is replaced by position.
        # This is necessary because QQuickWindow thinks of itself as
        # a top-level window always.
        mappedEvent = QMouseEvent(e.type(), e.position(), e.globalPosition(),
                                  e.button(), e.buttons(), e.modifiers())
        QCoreApplication.sendEvent(self.m_quickWindow, mappedEvent)

    def mouseReleaseEvent(self, e):
        mappedEvent = QMouseEvent(e.type(), e.position(), e.globalPosition(),
                                  e.button(), e.buttons(), e.modifiers())
        QCoreApplication.sendEvent(self.m_quickWindow, mappedEvent)



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test suite for QtCore.Qt namespace'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt


class QtNamespace(unittest.TestCase):
    '''Test case for accessing attributes from Qt namespace'''

    def testBasic(self):
        # Access to Qt namespace
        getattr(Qt, 'Horizontal')
        getattr(Qt, 'WindowMaximizeButtonHint')
        self.assertTrue(True)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QBrush'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import Qt
from PySide6.QtGui import QColor, QBrush, QConicalGradient

from helper.usesqapplication import UsesQApplication


class Constructor(UsesQApplication):
    '''Test case for constructor of QBrush'''

    def testQColor(self):
        # QBrush(QColor) constructor
        color = QColor('black')
        obj = QBrush(color)
        self.assertEqual(obj.color(), color)

        obj = QBrush(Qt.GlobalColor.blue)
        self.assertEqual(obj.color(), Qt.GlobalColor.blue)

    def testGradient(self):
        """Test type discovery on class hierarchies with non-virtual
           destructors by specifying a polymorphic-id-expression without
           polymorphic-name-function."""
        gradient = QConicalGradient()
        brush = QBrush(gradient)
        self.assertEqual(type(brush.gradient()), type(gradient))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtWidgets import (
    QWidget,
    QApplication,
    QMessageBox,
    QLineEdit,
    QProgressBar,
    QPushButton,
    QHBoxLayout,
    QVBoxLayout,
    QStyle,
    QFileDialog,
)
from PySide6.QtCore import QStandardPaths, QUrl, QFile, QSaveFile, QDir, QIODevice, Slot
from PySide6.QtNetwork import QNetworkReply, QNetworkRequest, QNetworkAccessManager
import sys


class DownloaderWidget(QWidget):
    """A widget to download a http file to a destination file"""

    def __init__(self, parent=None):
        super().__init__(parent)

        self.manager = QNetworkAccessManager(self)
        self.link_box = QLineEdit()
        self.dest_box = QLineEdit()
        self.progress_bar = QProgressBar()

        self.start_button = QPushButton("Start")
        self.abort_button = QPushButton("Abort")

        self.link_box.setPlaceholderText("Download Link ...")

        self._open_folder_action = self.dest_box.addAction(
            qApp.style().standardIcon(QStyle.StandardPixmap.SP_DirOpenIcon),  # noqa: F821
            QLineEdit.ActionPosition.TrailingPosition
        )
        self._open_folder_action.triggered.connect(self.on_open_folder)

        #  Current QFile
        self.file = None
        # Current QNetworkReply
        self.reply = None

        #  Default http url
        self.link_box.setText(
            "http://master.qt.io/archive/qt/6.0/6.0.1/single/qt-everywhere-src-6.0.1.zip"
        )

        #  Default destination dir
        self.dest_box.setText(
            QDir.fromNativeSeparators(
                QStandardPaths.writableLocation(QStandardPaths.StandardLocation.DownloadLocation)
            )
        )

        # buttons bar layout
        hlayout = QHBoxLayout()
        hlayout.addStretch()
        hlayout.addWidget(self.start_button)
        hlayout.addWidget(self.abort_button)

        # main layout
        vlayout = QVBoxLayout(self)
        vlayout.addWidget(self.link_box)
        vlayout.addWidget(self.dest_box)
        vlayout.addWidget(self.progress_bar)
        vlayout.addStretch()
        vlayout.addLayout(hlayout)

        self.resize(300, 100)

        self.start_button.clicked.connect(self.on_start)
        self.abort_button.clicked.connect(self.on_abort)

    @Slot()
    def on_start(self):
        """When user press start button"""

        #  http file
        url_file = QUrl(self.link_box.text())

        # destination file
        dest_path = QDir.fromNativeSeparators(self.dest_box.text().strip())
        dest_file = QDir(dest_path).filePath(url_file.fileName())

        # Ask a question if file already exists
        if QFile.exists(dest_file):
            ret = QMessageBox.question(
                self,
                "File exists",
                "Do you want to override the file ?",
                QMessageBox.Yes | QMessageBox.No,
            )
            if ret == QMessageBox.No:
                return

            QFile.remove(dest_file)

        self.start_button.setDisabled(True)
        # Create the file in write mode to append bytes
        self.file = QSaveFile(dest_file)

        if self.file.open(QIODevice.OpenModeFlag.WriteOnly):

            # Start a GET HTTP request
            self.reply = self.manager.get(QNetworkRequest(url_file))
            self.reply.downloadProgress.connect(self.on_progress)
            self.reply.finished.connect(self.on_finished)
            self.reply.readyRead.connect(self.on_ready_read)
            self.reply.errorOccurred.connect(self.on_error)
        else:
            error = self.file.errorString()
            print(f"Cannot open device: {error}")

    @Slot()
    def on_abort(self):
        """When user press abort button"""
        if self.reply:
            self.reply.abort()
            self.progress_bar.setValue(0)

        if self.file:
            self.file.cancelWriting()

        self.start_button.setDisabled(False)

    @Slot()
    def on_ready_read(self):
        """ Get available bytes and store them into the file"""
        if self.reply:
            if self.reply.error() == QNetworkReply.NetworkError.NoError:
                self.file.write(self.reply.readAll())

    @Slot()
    def on_finished(self):
        """ Delete reply and close the file"""
        if self.reply:
            self.reply.deleteLater()

        if self.file:
            self.file.commit()

        self.start_button.setDisabled(False)

    @Slot(int, int)
    def on_progress(self, bytesReceived: int, bytesTotal: int):
        """ Update progress bar"""
        self.progress_bar.setRange(0, bytesTotal)
        self.progress_bar.setValue(bytesReceived)

    @Slot(QNetworkReply.NetworkError)
    def on_error(self, code: QNetworkReply.NetworkError):
        """ Show a message if an error happen """
        if self.reply:
            QMessageBox.warning(self, "Error Occurred", self.reply.errorString())

    @Slot()
    def on_open_folder(self):

        dir_path = QFileDialog.getExistingDirectory(
            self, "Open Directory", QDir.homePath(), QFileDialog.Option.ShowDirsOnly
        )

        if dir_path:
            dest_dir = QDir(dir_path)
            self.dest_box.setText(QDir.fromNativeSeparators(dest_dir.path()))


if __name__ == "__main__":

    app = QApplication(sys.argv)

    w = DownloaderWidget()
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QTextBlockUserData, QTextCursor, QTextDocument


class MyData(QTextBlockUserData):
    def __init__(self, data):
        super().__init__()
        self.data = data

    def getMyNiceData(self):
        return self.data


class TestBug652(unittest.TestCase):
    """Segfault when using QTextBlock::setUserData due to missing ownership transfer"""
    def testIt(self):
        td = QTextDocument()
        tc = QTextCursor(td)
        tc.insertText("Hello world")
        heyHo = "hey ho!"
        tc.block().setUserData(MyData(heyHo))
        self.assertEqual(type(tc.block().userData()), MyData)
        self.assertEqual(tc.block().userData().getMyNiceData(), heyHo)

        del tc
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        tc = QTextCursor(td)
        blk = tc.block()
        self.assertEqual(type(blk.userData()), MyData)
        self.assertEqual(blk.userData().getMyNiceData(), heyHo)


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
sys.path.append(os.fspath(Path(__file__).resolve().parents[1] / "util"))
from init_paths import init_test_paths
init_test_paths()

from PySide6.QtWidgets import (QApplication, QGraphicsLinearLayout,
                               QGraphicsScene, QGraphicsView, QTextEdit)


app = QApplication([])

scene = QGraphicsScene()

# don't segfault due to lack of keepReferenceCall
textEdit = scene.addWidget(QTextEdit())

layout = QGraphicsLinearLayout()
layout.addItem(textEdit)

view = QGraphicsView(scene)
view.show()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import os
from PySide6.QtMultimedia import (QCameraFormat, QMediaFormat, QMediaRecorder,
                                  QVideoFrameFormat)
from PySide6.QtWidgets import QDialog

is_android = os.environ.get('ANDROID_ARGUMENT')

if is_android:
    from ui_videosettings_mobile import Ui_VideoSettingsUi
else:
    from ui_videosettings import Ui_VideoSettingsUi


def box_value(box):
    idx = box.currentIndex()
    return None if idx == -1 else box.itemData(idx)


def select_combo_box_item(box, value):
    idx = box.findData(value)
    if idx != -1:
        box.setCurrentIndex(idx)


def to_formatted_string(cameraFormat):
    pf = cameraFormat.pixelFormat()
    format_name = QVideoFrameFormat.pixelFormatToString(pf)
    w = cameraFormat.resolution().width()
    h = cameraFormat.resolution().height()
    min_rate = int(cameraFormat.minFrameRate())
    max_rate = int(cameraFormat.maxFrameRate())
    return f"{format_name} {w}x{h} {min_rate}-{max_rate}FPS"


class VideoSettings(QDialog):

    def __init__(self, mediaRecorder, parent=None):
        super().__init__(parent)

        self._media_recorder = mediaRecorder

        self.m_updatingFormats = False

        self._ui = Ui_VideoSettingsUi()
        self._ui.setupUi(self)

        # sample rate:
        audio_device = self._media_recorder.captureSession().audioInput().device()
        self._ui.audioSampleRateBox.setRange(audio_device.minimumSampleRate(),
                                             audio_device.maximumSampleRate())

        # camera format
        self._ui.videoFormatBox.addItem("Default camera format",
                                        QCameraFormat())

        camera = self._media_recorder.captureSession().camera()
        video_formats = camera.cameraDevice().videoFormats()

        for format in video_formats:
            self._ui.videoFormatBox.addItem(to_formatted_string(format), format)

        self._ui.videoFormatBox.currentIndexChanged.connect(self.video_format_changed)
        self.set_fps_range(camera.cameraFormat())

        self._ui.fpsSlider.valueChanged.connect(self._ui.fpsSpinBox.setValue)
        self._ui.fpsSpinBox.valueChanged.connect(self._ui.fpsSlider.setValue)

        self.update_formats_and_codecs()
        self._ui.audioCodecBox.currentIndexChanged.connect(self.update_formats_and_codecs)
        self._ui.videoCodecBox.currentIndexChanged.connect(self.update_formats_and_codecs)
        self._ui.containerFormatBox.currentIndexChanged.connect(self.update_formats_and_codecs)

        self._ui.qualitySlider.setRange(0, QMediaRecorder.VeryHighQuality.value)

        format = self._media_recorder.mediaFormat()
        select_combo_box_item(self._ui.containerFormatBox, format.fileFormat())
        select_combo_box_item(self._ui.audioCodecBox, format.audioCodec())
        select_combo_box_item(self._ui.videoCodecBox, format.videoCodec())

        self._ui.qualitySlider.setValue(self._media_recorder.quality().value)
        self._ui.audioSampleRateBox.setValue(self._media_recorder.audioSampleRate())
        select_combo_box_item(self._ui.videoFormatBox, camera.cameraFormat())

        self._ui.fpsSlider.setValue(self._media_recorder.videoFrameRate())
        self._ui.fpsSpinBox.setValue(self._media_recorder.videoFrameRate())

    def apply_settings(self):
        format = QMediaFormat()
        format.setFileFormat(box_value(self._ui.containerFormatBox))
        format.setAudioCodec(box_value(self._ui.audioCodecBox))
        format.setVideoCodec(box_value(self._ui.videoCodecBox))

        self._media_recorder.setMediaFormat(format)
        q = self._ui.qualitySlider.value()
        self._media_recorder.setQuality(QMediaRecorder.Quality(q))
        self._media_recorder.setAudioSampleRate(self._ui.audioSampleRateBox.value())

        camera_format = box_value(self._ui.videoFormatBox)
        self._media_recorder.setVideoResolution(camera_format.resolution())
        self._media_recorder.setVideoFrameRate(self._ui.fpsSlider.value())

        camera = self._media_recorder.captureSession().camera()
        camera.setCameraFormat(camera_format)

    def update_formats_and_codecs(self):
        if self.m_updatingFormats:
            return
        self.m_updatingFormats = True

        format = QMediaFormat()
        if self._ui.containerFormatBox.count():
            format.setFileFormat(box_value(self._ui.containerFormatBox))
        if self._ui.audioCodecBox.count():
            format.setAudioCodec(box_value(self._ui.audioCodecBox))
        if self._ui.videoCodecBox.count():
            format.setVideoCodec(box_value(self._ui.videoCodecBox))

        current_index = 0
        self._ui.audioCodecBox.clear()
        self._ui.audioCodecBox.addItem("Default audio codec",
                                       QMediaFormat.AudioCodec.Unspecified)
        for codec in format.supportedAudioCodecs(QMediaFormat.Encode):
            if codec == format.audioCodec():
                current_index = self._ui.audioCodecBox.count()
            desc = QMediaFormat.audioCodecDescription(codec)
            self._ui.audioCodecBox.addItem(desc, codec)

        self._ui.audioCodecBox.setCurrentIndex(current_index)

        current_index = 0
        self._ui.videoCodecBox.clear()
        self._ui.videoCodecBox.addItem("Default video codec",
                                       QMediaFormat.VideoCodec.Unspecified)
        for codec in format.supportedVideoCodecs(QMediaFormat.Encode):
            if codec == format.videoCodec():
                current_index = self._ui.videoCodecBox.count()
            desc = QMediaFormat.videoCodecDescription(codec)
            self._ui.videoCodecBox.addItem(desc, codec)

        self._ui.videoCodecBox.setCurrentIndex(current_index)

        current_index = 0
        self._ui.containerFormatBox.clear()
        self._ui.containerFormatBox.addItem("Default file format",
                                            QMediaFormat.UnspecifiedFormat)
        for container in format.supportedFileFormats(QMediaFormat.Encode):
            if container == format.fileFormat():
                current_index = self._ui.containerFormatBox.count()
            desc = QMediaFormat.fileFormatDescription(container)
            self._ui.containerFormatBox.addItem(desc, container)

        self._ui.containerFormatBox.setCurrentIndex(current_index)

        self.m_updatingFormats = False

    def video_format_changed(self):
        camera_format = box_value(self._ui.videoFormatBox)
        self.set_fps_range(camera_format)

    def set_fps_range(self, format):
        min_fr = format.minFrameRate()
        max_fr = format.maxFrameRate()
        self._ui.fpsSlider.setRange(min_fr, max_fr)
        self._ui.fpsSpinBox.setRange(min_fr, max_fr)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QTimer
from PySide6.QtSql import QSqlDatabase, QSqlQuery, QSqlTableModel


class TestBug1013 (unittest.TestCase):

    def someSlot(self, row, record):
        record.setValue(0, 2)
        self._wasCalled = True

    def testIt(self):
        app = QCoreApplication([])
        db = QSqlDatabase.addDatabase('QSQLITE')
        db.setDatabaseName(':memory:')
        db.open()
        query = QSqlQuery()
        query.exec('CREATE TABLE "foo" (id INT);')
        model = QSqlTableModel()
        model.setTable('foo')

        self._wasCalled = False
        model.primeInsert.connect(self.someSlot)
        model.select()
        QTimer.singleShot(0, lambda: model.insertRow(0) and app.quit())
        app.exec()
        self.assertTrue(self._wasCalled)
        self.assertEqual(model.data(model.index(0, 0)), 2)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Expression


class TestRichCompare(unittest.TestCase):

    def testIt(self):
        a = Expression(2)
        b = Expression(3)
        c = a + b
        d = a + c < b + a
        self.assertEqual(d.toString(), "((2+(2+3))<(3+2))")


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QMetaObject, Slot
from PySide6.QtWidgets import QApplication, QPushButton, QWidget


class MyObject(QWidget):
    def __init__(self, parent=None):
        QWidget.__init__(self, parent)
        self._method_called = False

    @Slot()
    def on_button_clicked(self):
        self._method_called = True


class AutoConnectionTest(unittest.TestCase):

    def testConnection(self):
        app = QApplication([])  # noqa: F841

        win = MyObject()
        btn = QPushButton("click", win)
        btn.setObjectName("button")
        QMetaObject.connectSlotsByName(win)
        btn.click()
        self.assertTrue(win._method_called)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for proper generation of constructor altered by conversion-rule tag.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import CtorConvRule


class TestCtorConvRule(unittest.TestCase):
    '''Simple test case for CtorConvRule'''

    def testCtorConvRule(self):
        '''Test CtorConvRule argument modification through conversion-rule tag.'''
        value = 123
        obj = CtorConvRule(value)
        self.assertEqual(obj.value(), value + 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtWidgets import QDoubleSpinBox, QGraphicsBlurEffect


class TestSignalConnection(UsesQApplication):
    def testFloatSignal(self):
        foo1 = QDoubleSpinBox()
        foo2 = QDoubleSpinBox()
        foo1.valueChanged[float].connect(foo2.setValue)
        foo2.valueChanged[float].connect(foo1.setValue)
        foo1.setValue(0.42)
        self.assertEqual(foo1.value(), foo2.value())

    def testQRealSignal(self):
        foo1 = QDoubleSpinBox()
        effect = QGraphicsBlurEffect()
        effect.blurRadiusChanged['qreal'].connect(foo1.setValue)  # check if qreal is a valid type
        effect.setBlurRadius(0.42)
        self.assertAlmostEqual(foo1.value(), effect.blurRadius())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from sample import HandleHolder
from shiboken6 import Shiboken


class TestHashFuncs (unittest.TestCase):

    def testIt(self):
        obj1 = HandleHolder()
        obj2 = HandleHolder()

        hash1 = hash(obj1)
        hash2 = hash(obj2)
        self.assertNotEqual(hash1, hash2)

        # Now invalidate the object and test its hash.  It shouldn't segfault.
        Shiboken.invalidate(obj1)

        hash1_2 = hash(obj1)
        self.assertEqual(hash1_2, hash1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QIPv6Address'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtNetwork import QIPv6Address


class QIPv6AddressGetItemTest(unittest.TestCase):
    def testLength(self):
        ip = QIPv6Address()
        self.assertEqual(len(ip), 16)

    def testSetItemNegativeIndex(self):
        ip = QIPv6Address()
        ip[-1] = 8
        self.assertEqual(ip[-1], 8)

    def testSetItemLargeIndex(self):
        ip = QIPv6Address()
        self.assertRaises(IndexError, ip.__setitem__, 32, 16)


if __name__ == '__main__':
    unittest.main()
